# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#                    Thomas Holder <thomas.holder@schrodinger.com>
#                    Sergei Izmailov <sergei.a.izmailov@gmail.com>
#                    Windell Oskay <windell@oskay.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=attribute-defined-outside-init
#
"""
Provide a way to load lxml attributes with an svg API on top.
"""

import random
from lxml import etree
import math

from ..deprecated import DeprecatedSvgMixin
from ..units import discover_unit
from ._selected import ElementList
from ..transforms import BoundingBox
from ..styles import StyleSheets

from ._base import BaseElement
from ._meta import StyleElement

if False: # pylint: disable=using-constant-test
    import typing # pylint: disable=unused-import


class SvgDocumentElement(DeprecatedSvgMixin, BaseElement):
    """Provide access to the document level svg functionality"""
    tag_name = 'svg'

    def _init(self):
        self.current_layer = None
        self.view_center = (0.0, 0.0)
        self.selection = ElementList(self)
        self.ids = {}

    def tostring(self):
        """Convert document to string"""
        return etree.tostring(etree.ElementTree(self))

    def get_ids(self):
        """Returns a set of unique document ids"""
        if not self.ids:
            self.ids = set(self.xpath('//@id'))
        return self.ids

    def get_unique_id(self, prefix, size=None):
        """Generate a new id from an existing old_id"""
        ids = self.get_ids()
        if size is None:
            size = max(math.ceil(math.log10(len(ids) or 1000))+1, 4)
        new_id = None
        _from = 10 ** size - 1
        _to = 10 ** size
        while new_id is None or new_id in ids:
            # Do not use randint because py2/3 incompatibility
            new_id = prefix + str(int(random.random() * _from - _to) + _to)
        self.ids.add(new_id)
        return new_id

    def get_page_bbox(self):
        """Gets the page dimensions as a bbox"""
        return BoundingBox((0, float(self.width)), (0, float(self.height)))

    def get_current_layer(self):
        """Returns the currently selected layer"""
        layer = self.getElementById(self.namedview.current_layer, 'svg:g')
        if layer is None:
            return self
        return layer

    def getElement(self, xpath):  # pylint: disable=invalid-name
        """Gets a single element from the given xpath or returns None"""
        return self.findone(xpath)

    def getElementById(self, eid, elm='*', literal=False): # pylint: disable=invalid-name
        """Get an element in this svg document by it's ID attribute"""
        if eid is not None and not literal:
            eid = eid.strip()[4:-1] if eid.startswith('url(') else eid
            eid = eid.lstrip('#')
        return self.getElement(f'//{elm}[@id="{eid}"]')

    def getElementByName(self, name, elm='*'): # pylint: disable=invalid-name
        """Get an element by it's inkscape:label (aka name)"""
        return self.getElement(f'//{elm}[@inkscape:label="{name}"]')

    def getElementsByClass(self, class_name): # pylint: disable=invalid-name
        """Get elements by it's class name"""
        from inkex.styles import ConditionalRule
        return self.xpath(ConditionalRule(f".{class_name}").to_xpath())

    def getElementsByHref(self, eid): # pylint: disable=invalid-name
        """Get elements by their href xlink attribute"""
        return self.xpath(f'//*[@xlink:href="#{eid}"]')

    def getElementsByStyleUrl(self, eid, style=None): # pylint: disable=invalid-name
        """Get elements by a style attribute url"""
        url = f"url(#{eid})"
        if style is not None:
            url = style + ":" + url
        return self.xpath(f'//*[contains(@style,"{url}")]')

    @property
    def name(self):
        """Returns the Document Name"""
        return self.get('sodipodi:docname', '')

    @property
    def namedview(self):
        """Return the sp namedview meta information element"""
        return self.get_or_create('//sodipodi:namedview', prepend=True)

    @property
    def metadata(self):
        """Return the svg metadata meta element container"""
        return self.get_or_create('//svg:metadata', prepend=True)

    @property
    def defs(self):
        """Return the svg defs meta element container"""
        return self.get_or_create('//svg:defs', prepend=True)

    def get_viewbox(self):
        """Parse and return the document's viewBox attribute"""
        try:
            ret = [float(unit) for unit in self.get('viewBox', '0').split()]
        except ValueError:
            ret = ''
        if len(ret) != 4:
            return [0, 0, 0, 0]
        return ret

    @property
    def width(self):  # getDocumentWidth(self):
        """Fault tolerance for lazily defined SVG"""
        return self.unittouu(self.get('width')) or self.get_viewbox()[2]

    @property
    def height(self):  # getDocumentHeight(self):
        """Returns a string corresponding to the height of the document, as
        defined in the SVG file. If it is not defined, returns the height
        as defined by the viewBox attribute. If viewBox is not defined,
        returns the string '0'."""
        return self.unittouu(self.get('height')) or self.get_viewbox()[3]

    @property
    def scale(self):
        """Return the ratio between the page width and the viewBox width"""
        try:
            scale_x = float(self.width) / float(self.get_viewbox()[2])
            scale_y = float(self.height) / float(self.get_viewbox()[3])
            return max([scale_x, scale_y])
        except (ValueError, ZeroDivisionError):
            return 1.0

    @property
    def unit(self):
        """Returns the unit used for in the SVG document.
        In the case the SVG document lacks an attribute that explicitly
        defines what units are used for SVG coordinates, it tries to calculate
        the unit from the SVG width and viewBox attributes.
        Defaults to 'px' units."""
        if not hasattr(self, '_unit'):
            self._unit = 'px' # Default is px
            viewbox = self.get_viewbox()
            if viewbox and set(viewbox) != {0}:
                self._unit = discover_unit(self.get('width'), viewbox[2], default='px')
        return self._unit

    @property
    def stylesheets(self):
        """Get all the stylesheets, bound together to one, (for reading)"""
        sheets = StyleSheets(self)
        for node in self.xpath('//svg:style'):
            sheets.append(node.stylesheet())
        return sheets

    @property
    def stylesheet(self):
        """Return the first stylesheet or create one if needed (for writing)"""
        for sheet in self.stylesheets:
            return sheet

        style_node = StyleElement()
        self.defs.append(style_node)
        return style_node.stylesheet()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import math

import inkex
from inkex.paths import Move, Line, Curve, ZoneClose, Arc, Path, Vert, Horz, TepidQuadratic, Quadratic, Smooth
from inkex.transforms import Vector2d
from inkex.bezier import beziertatslope, beziersplitatt


class Motion(inkex.EffectExtension):
    """Generate a motion path"""

    def add_arguments(self, pars):
        pars.add_argument("-a", "--angle", type=float, default=45.0, \
                          help="direction of the motion vector")
        pars.add_argument("-m", "--magnitude", type=float, default=100.0, \
                          help="magnitude of the motion vector")

    @staticmethod
    def makeface(last, segment, facegroup, delx, dely):
        """translate path segment along vector"""
        elem = facegroup.add(inkex.PathElement())

        npt = segment.translate([delx, dely])

        # reverse direction of path segment
        if isinstance(segment, Curve):
            rev = Curve(npt.x3, npt.y3, npt.x2, npt.y2,
                        last[0] + delx, last[1] + dely
                        )
        elif isinstance(segment, Line):
            rev = Line(last[0] + delx, last[1] + dely)
        else:
            raise RuntimeError("Unexpected segment type {}".format(type(segment)))

        elem.path = inkex.Path([
            Move(last[0], last[1]),
            segment,
            npt.to_line(Vector2d()),
            rev,
            ZoneClose(),
        ])

    def effect(self):
        delx = math.cos(math.radians(self.options.angle)) * self.options.magnitude
        dely = math.sin(math.radians(self.options.angle)) * self.options.magnitude
        for node in self.svg.selection.filter(inkex.PathElement):
            group = node.getparent().add(inkex.Group())
            facegroup = group.add(inkex.Group())
            group.append(node)

            if node.transform:
                group.transform = node.transform
                node.transform = None

            facegroup.style = node.style

            for cmd_proxy in node.path.to_absolute().proxy_iterator():
                self.process_segment(cmd_proxy, facegroup, delx, dely)

    @staticmethod
    def process_segment(cmd_proxy, facegroup, delx, dely):
        """Process each segments"""

        segments = []
        if isinstance(cmd_proxy.command, (Curve, Smooth, TepidQuadratic, Quadratic, Arc)):
            prev = cmd_proxy.previous_end_point
            for curve in cmd_proxy.to_curves():
                bez = [prev] + curve.to_bez()
                prev = curve.end_point(cmd_proxy.first_point, prev)
                tees = [t for t in beziertatslope(bez, (dely, delx)) if 0 < t < 1]
                tees.sort()
                if len(tees) == 1:
                    one, two = beziersplitatt(bez, tees[0])
                    segments.append(Curve(*(one[1] + one[2] + one[3])))
                    segments.append(Curve(*(two[1] + two[2] + two[3])))
                elif len(tees) == 2:
                    one, two = beziersplitatt(bez, tees[0])
                    two, three = beziersplitatt(two, tees[1])
                    segments.append(Curve(*(one[1] + one[2] + one[3])))
                    segments.append(Curve(*(two[1] + two[2] + two[3])))
                    segments.append(Curve(*(three[1] + three[2] + three[3])))
                else:
                    segments.append(curve)
        elif isinstance(cmd_proxy.command, (Line, Curve)):
            segments.append(cmd_proxy.command)
        elif isinstance(cmd_proxy.command, ZoneClose):
            segments.append(Line(cmd_proxy.first_point.x, cmd_proxy.first_point.y))
        elif isinstance(cmd_proxy.command, (Vert, Horz)):
            segments.append(cmd_proxy.command.to_line(cmd_proxy.end_point))

        for seg in Path([Move(*cmd_proxy.previous_end_point)] + segments).proxy_iterator():
            if isinstance(seg.command, Move): continue
            Motion.makeface(seg.previous_end_point, seg.command, facegroup, delx, dely)


if __name__ == '__main__':
    Motion().run()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#                    Sergei Izmailov <sergei.a.izmailov@gmail.com>
#                    Ryan Jarvis <ryan@shopboxretail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=arguments-differ
"""
Interface for all group based elements such as Groups, Use, Markers etc.
"""

from lxml import etree # pylint: disable=unused-import

from ..paths import Path
from ..transforms import Transform

from ._utils import addNS
from ._base import ShapeElement

try:
    from typing import Optional  # pylint: disable=unused-import
except ImportError:
    pass

class GroupBase(ShapeElement):
    """Base Group element"""
    def get_path(self):
        ret = Path()
        for child in self:
            if isinstance(child, ShapeElement):
                ret += child.path.transform(child.transform)
        return ret

    def shape_box(self, transform=None):
        bbox = None
        effective_transform = Transform(transform) * self.transform
        for child in self:
            if isinstance(child, ShapeElement):
                child_bbox = child.bounding_box(transform=effective_transform)
                if child_bbox is not None:
                    bbox += child_bbox
        return bbox


class Group(GroupBase):
    """Any group element (layer or regular group)"""
    tag_name = 'g'

    @classmethod
    def new(cls, label, *children, **attrs):
        attrs['inkscape:label'] = label
        return super().new(*children, **attrs)


    def effective_style(self):
        """A blend of each child's style mixed together (last child wins)"""
        style = self.style
        for child in self:
            style.update(child.effective_style())
        return style

    @property
    def groupmode(self):
        """Return the type of group this is"""
        return self.get('inkscape:groupmode', 'group')


class Layer(Group):
    """Inkscape extension of svg:g"""

    def _init(self):
        self.set('inkscape:groupmode', 'layer')

    @classmethod
    def _is_class_element(cls, el):
        # type: (etree.Element) -> bool
        return el.attrib.get(addNS('inkscape:groupmode'), None) == "layer"


class Anchor(GroupBase):
    """An anchor or link tag"""
    tag_name = 'a'

    @classmethod
    def new(cls, href, *children, **attrs):
        attrs['xlink:href'] = href
        return super().new(*children, **attrs)


class ClipPath(GroupBase):
    """A path used to clip objects"""
    tag_name = 'clipPath'


class Marker(GroupBase):
    """The <marker> element defines the graphic that is to be used for drawing arrowheads
     or polymarkers on a given <path>, <line>, <polyline> or <polygon> element."""
    tag_name = 'marker'



#
# Copyright (C) 2019 Thomas Holder
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=too-few-public-methods
#
"""
Comparison filters for use with the ComparisonMixin.

Each filter should be initialised in the list of
filters that are being used.

.. code-block:: python
.. compare_filters = [
..    CompareNumericFuzzy(),
..    CompareOrderIndependentLines(option=yes),
.. ]
"""

import re
from ..utils import to_bytes

class Compare:
    """
    Comparison base class, this acts as a passthrough unless
    the filter staticmethod is overwritten.
    """
    def __init__(self, **options):
        self.options = options

    def __call__(self, content):
        return self.filter(content)

    @staticmethod
    def filter(contents):
        """Replace this filter method with your own filtering"""
        return contents

class CompareNumericFuzzy(Compare):
    """
    Turn all numbers into shorter standard formats

    1.2345678 -> 1.2346
    1.2300 -> 1.23, 50.0000 -> 50.0
    50.0 -> 50
    """
    @staticmethod
    def filter(contents):
        func = lambda m: b'%.3f' % (float(m.group(0)))
        contents = re.sub(br'\d+\.\d+', func, contents)
        contents = re.sub(br'(\d\.\d+?)0+\b', br'\1', contents)
        contents = re.sub(br'(\d)\.0+(?=\D|\b)', br'\1', contents)
        return contents

class CompareWithoutIds(Compare):
    """Remove all ids from the svg"""
    @staticmethod
    def filter(contents):
        return re.sub(br' id="([^"]*)"', b'', contents)

class CompareWithPathSpace(Compare):
    """Make sure that path segment commands have spaces around them"""
    @staticmethod
    def filter(contents):
        def func(match):
            """We've found a path command, process it"""
            new = re.sub(br'\s*([LZMHVCSQTAatqscvhmzl])\s*', br' \1 ', match.group(1))
            return b' d="' + new.replace(b',', b' ') + b'"'
        return re.sub(br' d="([^"]*)"', func, contents)

class CompareSize(Compare):
    """Compare the length of the contents instead of the contents"""
    @staticmethod
    def filter(contents):
        return len(contents)

class CompareOrderIndependentBytes(Compare):
    """Take all the bytes and sort them"""
    @staticmethod
    def filter(contents):
        return b"".join([bytes(i) for i in sorted(contents)])

class CompareOrderIndependentLines(Compare):
    """Take all the lines and sort them"""
    @staticmethod
    def filter(contents):
        return b"\n".join(sorted(contents.splitlines()))

class CompareOrderIndependentStyle(Compare):
    """Take all styles and sort the results"""
    @staticmethod
    def filter(contents):
        contents = CompareNumericFuzzy.filter(contents)
        def func(match):
            """Search and replace function for sorting"""
            sty = b';'.join(sorted(match.group(1).split(b';')))
            return b'style="%s"' % (sty,)
        return re.sub(br'style="([^"]*)"', func, contents)

class CompareOrderIndependentStyleAndPath(Compare):
    """Take all styles and paths and sort them both"""
    @staticmethod
    def filter(contents):
        contents = CompareOrderIndependentStyle.filter(contents)
        def func(match):
            """Search and replace function for sorting"""
            path = b'X'.join(sorted(re.split(br'[A-Z]', match.group(1))))
            return b'd="%s"' % (path,)
        return re.sub(br'\bd="([^"]*)"', func, contents)

class CompareOrderIndependentTags(Compare):
    """Sorts all the XML tags"""
    @staticmethod
    def filter(contents):
        return b"\n".join(sorted(re.split(br'>\s*<', contents)))

class CompareReplacement(Compare):
    """Replace pieces to make output more comparable"""
    def __init__(self, *replacements):
        self.deltas = replacements
        super().__init__()

    def filter(self, contents):
        contents = to_bytes(contents)
        for _from, _to in self.deltas:
            contents = contents.replace(to_bytes(_from), to_bytes(_to))
        return contents



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""Straintens path segments (but doesn't turn them into lines)"""
import inkex

from inkex.bezier import percent_point

class SegmentStraightener(inkex.EffectExtension):
    """Make segments straiter"""
    def add_arguments(self, pars):
        pars.add_argument("-p", "--percent", type=float, default=10.0,\
            help="move curve handles PERCENT percent closer to a straight line")
        pars.add_argument("-b", "--behavior", type=int, default=1,\
            help="straightening behavior for cubic segments")

    def effect(self):
        for node in self.svg.selection.get(inkex.PathElement):
            path = node.path.to_arrays()
            last = []
            sub_start = []
            for cmd, params in path:
                if cmd == 'C':
                    if self.options.behavior <= 1:
                        #shorten handles towards end points
                        params[:2] = percent_point(params[:2], last[:], self.options.percent)
                        params[2:4] = percent_point(params[2:4], params[-2:], self.options.percent)
                    else:
                        #shorten handles towards thirds of the segment
                        dest1 = percent_point(last[:], params[-2:], 33.3)
                        dest2 = percent_point(params[-2:], last[:], 33.3)
                        params[:2] = percent_point(params[:2], dest1[:], self.options.percent)
                        params[2:4] = percent_point(params[2:4], dest2[:], self.options.percent)
                elif cmd == 'Q':
                    dest = percent_point(last[:], params[-2:], 50)
                    params[:2] = percent_point(params[:2], dest, self.options.percent)
                if cmd == 'M':
                    sub_start = params[-2:]
                if cmd == 'Z':
                    last = sub_start[:]
                else:
                    last = params[-2:]
            node.path = path

if __name__ == '__main__':
    SegmentStraightener().run()



# coding=utf-8
# COPYRIGHT
"""DOCSTRING"""

import inkex
from inkex.colors import SVG_COLOR as svgcolors
from inkex.deprecated import deprecate

@deprecate
def parseStyle(s):
    """dict(inkex.Style.parse_str(s))"""
    return dict(inkex.Style.parse_str(s))

@deprecate
def formatStyle(a):
    """str(inkex.Style(a))"""
    return str(inkex.Style(a))

@deprecate
def isColor(c):
    """inkex.colors.is_color(c)"""
    return inkex.colors.is_color(c)

@deprecate
def parseColor(c):
    """inkex.Color(c).to_rgb()"""
    return tuple(inkex.Color(c).to_rgb())

@deprecate
def formatColoria(a):
    """str(inkex.Color(a))"""
    return str(inkex.Color(a))

@deprecate
def formatColorfa(a):
    """str(inkex.Color(a))"""
    return str(inkex.Color(a))

@deprecate
def formatColor3i(r,g,b):
    """str(inkex.Color((r, g, b)))"""
    return str(inkex.Color((r, g, b)))

@deprecate
def formatColor3f(r,g,b):
    """str(inkex.Color((r, g, b)))"""
    return str(inkex.Color((r, g, b)))



#!/usr/bin/env python
"""Convert text to upper case"""

import inkex

class Uppercase(inkex.TextExtension):
    """To upper case"""
    def process_chardata(self, text):
        return text.upper()

if __name__ == '__main__':
    Uppercase().run()



#!/usr/bin/env python
"""Extension for removing the colour red from selected objects"""

import inkex

class RemoveRed(inkex.ColorExtension):
    """Remove red color from selected objects"""
    def modify_color(self, name, color):
        return inkex.Color([0, color.green, color.blue])

if __name__ == '__main__':
    RemoveRed().run()



# coding=utf-8
#
# Copyright (C) 2009 Aaron C Spike
#               2010 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
#
"""
Python barcode renderer for EAN5 barcodes. Designed for use with Inkscape.
"""

from .BaseEan import EanBarcode

FAMS = ['11000', '10100', '10010', '10001', '01100',
        '00110', '00011', '01010', '01001', '00101']
START = '01011'


class Ean5(EanBarcode):
    """Provide an Ean5 barcode generator"""
    name = 'ean5'
    length = 5

    def _encode(self, num, guide=False):
        self.text = ' '.join(self.space(num))
        family = sum([int(n) * int(m) for n, m in zip(num, '39393')]) % 10
        return START + '01'.join(self.encode_interleaved(family, num, FAMS))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Martin Owens <doctormo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Useful utilities specifically for elements (that aren't base classes)
"""

from collections import defaultdict
import re

# a dictionary of all of the xmlns prefixes in a standard inkscape doc
NSS = {
    'sodipodi': 'http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd',
    'cc': 'http://creativecommons.org/ns#',
    'ccOLD': 'http://web.resource.org/cc/',
    'svg': 'http://www.w3.org/2000/svg',
    'dc': 'http://purl.org/dc/elements/1.1/',
    'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
    'inkscape': 'http://www.inkscape.org/namespaces/inkscape',
    'xlink': 'http://www.w3.org/1999/xlink',
    'xml': 'http://www.w3.org/XML/1998/namespace'
}
SSN = dict((b, a) for (a, b) in NSS.items())

def addNS(tag, ns=None):  # pylint: disable=invalid-name
    """Add a known namespace to a name for use with lxml"""
    if tag.startswith('{') and ns: 
        _, tag = removeNS(tag)
    if not tag.startswith('{'):
        tag = tag.replace('__', ':')
        if ':' in tag:
            (ns, tag) = tag.rsplit(':', 1)
        if ns in NSS:
            ns = NSS[ns]
        if ns is not None:
            return "{%s}%s" % (ns, tag)
    return tag 


def removeNS(name):  # pylint: disable=invalid-name
    """The reverse of addNS, finds any namespace and returns tuple (ns, tag)"""
    if name[0] == '{':
        (url, tag) = name[1:].split('}', 1)
        return SSN.get(url, 'svg'), tag 
    if ':' in name:
        return name.rsplit(':', 1)
    return 'svg', name

def splitNS(name): # pylint: disable=invalid-name
    """Like removeNS, but returns a url instead of a prefix"""
    (prefix, tag) = removeNS(name)
    return (NSS[prefix], tag)

def natural_sort_key(s, _nsre=re.compile('([0-9]+)')):
    """Helper for a natural sort, see
    https://stackoverflow.com/a/16090640/3298143"""
    return [int(text) if text.isdigit() else text.lower()
            for text in _nsre.split(s)]   

class ChildToProperty(property):
    """Use when you have a singleton child element who's text
       content is the canonical value for the property"""
    def __init__(self, tag, prepend=False):
        self.tag = tag
        self.prepend = prepend

    def __get__(self, obj, klass=None):
        elem = obj.findone(self.tag)
        return elem.text if elem is not None else None

    def __set__(self, obj, value):
        elem = obj.get_or_create(self.tag, prepend=self.prepend)
        elem.text = value

    def __delete__(self, obj):
        obj.remove_all(self.tag)

    @property
    def __doc__(self):
        return f"Get, set or delete the {self.tag} property."

class CloningVat(object):
    """
    When modifying defs, sometimes we want to know if every backlink would have
    needed changing, or it was just some of them.

    This tracks the def elements, their promises and creates clones if needed.
    """
    def __init__(self, svg):
        self.svg = svg
        self.tracks = defaultdict(set)
        self.set_ids = defaultdict(list)

    def track(self, elem, parent, set_id=None, **kwargs):
        """Track the element and connected parent"""
        elem_id = elem.get('id')
        parent_id = parent.get('id')
        self.tracks[elem_id].add(parent_id)
        self.set_ids[elem_id].append((set_id, kwargs))

    def process(self, process, types=(), make_clones=True, **kwargs):
        """
        Process each tracked item if the backlinks match the parents

        Optionally make clones, process the clone and set the new id.
        """
        for elem_id in list(self.tracks):
            parents = self.tracks[elem_id]
            elem = self.svg.getElementById(elem_id)
            backlinks = set([blk.get('id') for blk in elem.backlinks(*types)])
            if backlinks == parents:
                # No need to clone, we're processing on-behalf of all parents
                process(elem, **kwargs)
            elif make_clones:
                clone = elem.copy()
                elem.getparent().append(clone)
                clone.set_random_id()
                for update, upkw in self.set_ids.get(elem_id, ()):
                    update(elem.get('id'), clone.get('id'), **upkw)
                process(clone, **kwargs)




#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2019 Marc Jeanmougin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
"""
Generate Latex via a PDF using pdflatex
"""

import os

import inkex
from inkex.base import TempDirMixin
from inkex.command import call, inkscape
from inkex import load_svg, ShapeElement, Defs

class PdfLatex(TempDirMixin, inkex.GenerateExtension):
    """
    Use pdflatex to generate LaTeX, this whole hack is required because
    we don't want to open a LaTeX document as a document, but as a
    generated fragment (like import, but done manually).
    """
    def add_arguments(self, pars):
        pars.add_argument('--formule', type=str, default='')
        pars.add_argument('--packages', type=str, default='')

    def generate(self):
        tex_file = os.path.join(self.tempdir, 'input.tex')
        pdf_file = os.path.join(self.tempdir, 'input.pdf') # Auto-generate by pdflatex
        svg_file = os.path.join(self.tempdir, 'output.svg')

        with open(tex_file, 'w') as fhl:
            self.write_latex(fhl)

        call('pdflatex', tex_file,\
            output_directory=self.tempdir,\
            halt_on_error=True, oldie=True)

        inkscape(pdf_file, export_filename=svg_file, pdf_page=1,
                 pdf_poppler=True, export_type="svg")

        if not os.path.isfile(svg_file):
            fn = os.path.basename(svg_file)
            if os.path.isfile(fn):
                # Inkscape bug detected, file got saved wrong
                svg_file = fn

        with open(svg_file, 'r') as fhl:
            svg = load_svg(fhl).getroot()
            svg.set_random_ids(backlinks=True)
            for child in svg:
                if isinstance(child, ShapeElement):
                    yield child
                elif isinstance(child, Defs):
                    for def_child in child:
                        #def_child.set_random_id()
                        self.svg.defs.append(def_child)

    def write_latex(self, stream):
        """Takes a forumle and wraps it in latex"""
        stream.write(r"""%% processed with pdflatex.py
\documentclass{minimal}
\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{amsfonts}
""")
        for package in self.options.packages.split(','):
            if package:
                stream.write('\\usepackage{{{}}}\n'.format(package))
        stream.write("\n\\begin{document}\n")
        stream.write(self.options.formule)
        stream.write("\n\\end{document}\n")
        stream.flush()

if __name__ == '__main__':
    PdfLatex().run()



#!/usr/bin/env python
# coding=utf-8
# Copyright (C) 2011 Karlisson Bezerra <contact@hacktoon.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Save an SVG file into an html canvas file.
"""

import inkex

import ink2canvas_lib.svg as svg
from ink2canvas_lib.canvas import Canvas

class Html5Canvas(inkex.OutputExtension):
    """Creates a canvas output"""
    def save(self, stream):
        svg_root = self.document.getroot()
        width = self.svg.unittouu(svg_root.get("width"))
        height = self.svg.unittouu(svg_root.get("height"))
        canvas = Canvas(self, width, height)
        self.walk_tree(svg_root, canvas)
        stream.write(canvas.output().encode('utf-8'))

    def get_gradient_defs(self, elem):
        """Return the gradient information"""
        url_id = elem.get_gradient_href()
        # get the gradient element
        gradient = self.svg.getElementById(url_id)
        # get the color stops
        gstops = gradient.href
        colors = []
        for stop in gstops:
            colors.append(stop.get("style"))
        if gradient.get("r"):
            return svg.RadialGradientDef(gradient, colors)
        return svg.LinearGradientDef(gradient, colors)

    @staticmethod
    def _shape_from_node(node, canvas):
        """
        Make a canvas shape object for the given node. Returns `None` if
        the node is not an SVG shape element.
        @rtype svg.AbstractShape or NoneType
        """
        prefix, _brace_, command = node.tag.partition('}')
        if prefix != '{http://www.w3.org/2000/svg':
            return None

        # makes pylint happy
        assert _brace_ == '}'

        cls = getattr(svg, command.capitalize(), None)

        if not (isinstance(cls, type) and issubclass(cls, svg.AbstractShape)):
            return None

        return cls(command, node, canvas)

    def walk_tree(self, root, canvas):
        """Walk throug the whole svg tree"""
        for node in root:
            elem = self._shape_from_node(node, canvas)
            if elem is None:
                continue
            gradient = None
            if elem.has_gradient():
                gradient = self.get_gradient_defs(elem)
            elem.start(gradient)
            try:
                elem.draw()
            except ValueError as error: # print out the reason if any element can not be exported
                canvas.write("// " + str(error))
                continue
            self.walk_tree(node, canvas)
            elem.end()


if __name__ == "__main__":
    Html5Canvas().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007 Terry Brown, terry_n_brown@yahoo.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
from __future__ import absolute_import, unicode_literals

import inkex
from inkex import Use, Rectangle
from inkex.base import SvgOutputMixin

class Nup(inkex.OutputExtension, SvgOutputMixin):
    """N-up Layout generator"""
    def add_arguments(self, pars):
        pars.add_argument('--unit', default='px')
        pars.add_argument('--rows', type=int, default=2)
        pars.add_argument('--cols', type=int, default=2)
        pars.add_argument('--paddingTop', type=float)
        pars.add_argument('--paddingBottom', type=float)
        pars.add_argument('--paddingLeft', type=float)
        pars.add_argument('--paddingRight', type=float)
        pars.add_argument('--marginTop', type=float)
        pars.add_argument('--marginBottom', type=float)
        pars.add_argument('--marginLeft', type=float)
        pars.add_argument('--marginRight', type=float)
        pars.add_argument('--pgMarginTop', type=float)
        pars.add_argument('--pgMarginBottom', type=float)
        pars.add_argument('--pgMarginLeft', type=float)
        pars.add_argument('--pgMarginRight', type=float)
        pars.add_argument('--pgSizeX', type=float)
        pars.add_argument('--pgSizeY', type=float)
        pars.add_argument('--sizeX', type=float)
        pars.add_argument('--sizeY', type=float)
        pars.add_argument('--calculateSize', type=inkex.Boolean, default=True)
        pars.add_argument('--showHolder', type=inkex.Boolean, default=True)
        pars.add_argument('--showCrosses', type=inkex.Boolean, default=True)
        pars.add_argument('--showInner', type=inkex.Boolean, default=True)
        pars.add_argument('--showOuter', type=inkex.Boolean, default=False)
        pars.add_argument('--showInnerBox', type=inkex.Boolean, default=False)
        pars.add_argument('--showOuterBox', type=inkex.Boolean, default=False)
        pars.add_argument('--tab')

    def save(self, stream):
        show_list = []
        for i in ['showHolder', 'showCrosses', 'showInner', 'showOuter',
                  'showInnerBox', 'showOuterBox', ]:
            if getattr(self.options, i):
                show_list.append(i.lower().replace('show', ''))
        opt = self.options
        ret = self.generate_nup(
            unit=opt.unit,
            pgSize=(opt.pgSizeX, opt.pgSizeY),
            pgMargin=(opt.pgMarginTop, opt.pgMarginRight, opt.pgMarginBottom, opt.pgMarginLeft),
            num=(opt.rows, opt.cols),
            calculateSize=opt.calculateSize,
            size=(opt.sizeX, opt.sizeY),
            margin=(opt.marginTop, opt.marginRight, opt.marginBottom, opt.marginLeft),
            padding=(opt.paddingTop, opt.paddingRight, opt.paddingBottom, opt.paddingLeft),
            show=show_list,
        )
        if ret:
            stream.write(ret)

    def expandTuple(self, unit, x, length=4):
        try:
            iter(x)
        except:
            return None

        if len(x) != length:
            x *= 2
        if len(x) != length:
            raise Exception("expandTuple: requires 2 or 4 item tuple")
        try:
            return tuple(map(lambda ev: (self.svg.unittouu(str(eval(str(ev))) + unit) / self.svg.unittouu('1px')), x))
        except:
            return None

    def generate_nup(self,
                     unit="px",
                     pgSize=("8.5*96", "11*96"),
                     pgMargin=(0, 0),
                     pgPadding=(0, 0),
                     num=(2, 2),
                     calculateSize=True,
                     size=None,
                     margin=(0, 0),
                     padding=(20, 20),
                     show=['default'],
                    ):
        """Generate the SVG.  Inputs are run through 'eval(str(x))' so you can use
    '8.5*72' instead of 612.  Margin / padding dimension tuples can be
    (top & bottom, left & right) or (top, right, bottom, left).

    Keyword arguments:
    pgSize -- page size, width x height
    pgMargin -- extra space around each page
    pgPadding -- added to pgMargin
    n -- rows x cols
    size -- override calculated size, width x height
    margin -- white space around each piece
    padding -- inner padding for each piece
    show -- list of keywords indicating what to show
            - 'crosses' - cutting guides
            - 'inner' - inner boundary
            - 'outer' - outer boundary
    """

        if 'default' in show:
            show = set(show).union(['inner', 'innerbox', 'holder', 'crosses'])

        pgMargin = self.expandTuple(unit, pgMargin)
        pgPadding = self.expandTuple(unit, pgPadding)
        margin = self.expandTuple(unit, margin)
        padding = self.expandTuple(unit, padding)

        pgSize = self.expandTuple(unit, pgSize, length=2)
        #    num = tuple(map(lambda ev: eval(str(ev)), num))

        if not pgMargin or not pgPadding:
            return inkex.errormsg("No padding or margin available.")

        page_edge = list(map(sum, zip(pgMargin, pgPadding)))

        top, right, bottom, left = 0, 1, 2, 3
        width, height = 0, 1
        rows, cols = 0, 1
        size = self.expandTuple(unit, size, length=2)
        if size is None or calculateSize or len(size) < 2 or size[0] == 0 or size[1] == 0:
            size = ((pgSize[width]
                     - page_edge[left] - page_edge[right]
                     - num[cols] * (margin[left] + margin[right])) / num[cols],
                    (pgSize[height]
                     - page_edge[top] - page_edge[bottom]
                     - num[rows] * (margin[top] + margin[bottom])) / num[rows]
                   )
        else:
            size = self.expandTuple(unit, size, length=2)

        # sep is separation between same points on pieces
        sep = (size[width] + margin[right] + margin[left],
               size[height] + margin[top] + margin[bottom])

        style = 'stroke:#000000;stroke-opacity:1;fill:none;fill-opacity:1;'

        padbox = Rectangle(
            x=str(page_edge[left] + margin[left] + padding[left]),
            y=str(page_edge[top] + margin[top] + padding[top]),
            width=str(size[width] - padding[left] - padding[right]),
            height=str(size[height] - padding[top] - padding[bottom]),
            style=style,
        )
        margbox = Rectangle(
            x=str(page_edge[left] + margin[left]),
            y=str(page_edge[top] + margin[top]),
            width=str(size[width]),
            height=str(size[height]),
            style=style,
        )

        doc = self.get_template(width=pgSize[width], height=pgSize[height])
        svg = doc.getroot()

        def make_clones(under, to):
            for row in range(0, num[rows]):
                for col in range(0, num[cols]):
                    if row == 0 and col == 0:
                        continue
                    use = under.add(Use())
                    use.set('xlink:href', '#' + to)
                    use.transform.add_translate(col * sep[width], row * sep[height])

        # guidelayer #####################################################
        if {'inner', 'outer'}.intersection(show):
            layer = svg.add(inkex.Layer.new('Guide Layer'))
            if 'inner' in show:
                ibox = layer.add(padbox.copy())
                ibox.style['stroke'] = '#8080ff'
                ibox.set('id', 'innerguide')
                make_clones(layer, 'innerguide')
            if 'outer' in show:
                obox = layer.add(margbox.copy())
                obox.style['stroke'] = '#8080ff'
                obox.set('id', 'outerguide')
                make_clones(layer, 'outerguide')

        # crosslayer #####################################################
        if {'crosses'}.intersection(show):
            layer = svg.add(inkex.Layer.new('Cut Layer'))

            if 'crosses' in show:
                crosslen = 12
                group = layer.add(inkex.Group(id='cross'))
                x, y = 0, 0
                path = 'M%f %f' % (x + page_edge[left] + margin[left],
                                   y + page_edge[top] + margin[top] - crosslen)
                path += ' L%f %f' % (x + page_edge[left] + margin[left],
                                     y + page_edge[top] + margin[top] + crosslen)
                path += ' M%f %f' % (x + page_edge[left] + margin[left] - crosslen,
                                     y + page_edge[top] + margin[top])
                path += ' L%f %f' % (x + page_edge[left] + margin[left] + crosslen,
                                     y + page_edge[top] + margin[top])
                group.add(inkex.PathElement(style=style + 'stroke-width:0.05',
                                            d=path, id='crossmarker'))
                for row in 0, 1:
                    for col in 0, 1:
                        if row or col:
                            cln = group.add(Use())
                            cln.set('xlink:href', '#crossmarker')
                            cln.transform.add_translate(col * size[width], row * size[height])
                make_clones(layer, 'cross')

        # clonelayer #####################################################
        layer = svg.add(inkex.Layer.new('Clone Layer'))
        make_clones(layer, 'main')

        # mainlayer ######################################################
        layer = svg.add(inkex.Layer.new('Main Layer'))
        group = layer.add(inkex.Group(id='main'))

        if 'innerbox' in show:
            group.add(padbox)
        if 'outerbox' in show:
            group.add(margbox)
        if 'holder' in show:
            x, y = (page_edge[left] + margin[left] + padding[left],
                    page_edge[top] + margin[top] + padding[top])
            w, h = (size[width] - padding[left] - padding[right],
                    size[height] - padding[top] - padding[bottom])
            path = 'M{:f} {:f}'.format(x + w / 2., y)
            path += ' L{:f} {:f}'.format(x + w, y + h / 2.)
            path += ' L{:f} {:f}'.format(x + w / 2., y + h)
            path += ' L{:f} {:f}'.format(x, y + h / 2.)
            path += ' Z'
            group.add(inkex.PathElement(style=style, d=path))

        return svg.tostring()


if __name__ == '__main__':
    Nup().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2006 Jean-Francois Barraud, barraud@math.univ-lille1.fr
#               2021 Jonathan Neuhauser, jonathan.neuhauser@outlook.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
# barraud@math.univ-lille1.fr
"""
This script scatters an object (the pattern) along other paths (skeletons)...
The topmost selected object is the pattern
the other selected ones are the skeletons.

No deformation is applied to the pattern itself.
"""
import random
import math

import inkex
from inkex import bezier, Transform, BoundingBox, Group, Use
from inkex.localization import inkex_gettext as _

import pathmodifier

class PathScatter(pathmodifier.Diffeo):
    def __init__(self):
        super(PathScatter, self).__init__()
        self.arg_parser.add_argument("-n", "--noffset", type=float, dest="noffset", default=0.0, help="normal offset")
        self.arg_parser.add_argument("-t", "--toffset", type=float, dest="toffset", default=0.0, help="tangential offset")
        self.arg_parser.add_argument("-g", "--grouppick", type=inkex.Boolean, dest="grouppick", default=False,
                                     help="if pattern is a group then randomly pick group members")
        self.arg_parser.add_argument("-m", "--pickmode", type=str, dest="pickmode", default="rand",
                                     help="group pick mode (rand=random seq=sequentially)")
        self.arg_parser.add_argument("-f", "--follow", type=inkex.Boolean, dest="follow", default=True,
                                     help="choose between wave or snake effect")
        self.arg_parser.add_argument("-s", "--stretch", type=inkex.Boolean, dest="stretch", default=True,
                                     help="repeat the path to fit deformer's length")
        self.arg_parser.add_argument("-p", "--space", type=float, dest="space", default=0.0)
        self.arg_parser.add_argument("-r", "--rotate", type=inkex.Boolean, dest="vertical", default=False,
                                     help="reference path is vertical")
        self.arg_parser.add_argument("-c", "--copymode", type=str, dest="copymode", default="move",
                                     help="""How the pattern is duplicated. Default: 'move', 
                                     Options: 'clone', 'duplicate', 'move'""")
        self.arg_parser.add_argument("--tab", type=str, dest="tab",
                                     help="The selected UI-tab when OK was pressed")

    def localTransformAt(self, s, skelcomp, lengths, isclosed, follow=True):
        """
        receives a length, and returns the corresponding point and tangent of skelcomp
        if follow is set to false, returns only the translation
        """
        i, t = self.lengthtotime(s, lengths, isclosed)
        if i == len(skelcomp) - 1:
            x, y = bezier.between_point(skelcomp[i - 1], skelcomp[i], 1 + t)
            dx = (skelcomp[i][0] - skelcomp[i - 1][0]) / lengths[-1]
            dy = (skelcomp[i][1] - skelcomp[i - 1][1]) / lengths[-1]
        else:
            x, y = bezier.between_point(skelcomp[i], skelcomp[i + 1], t)
            dx = (skelcomp[i + 1][0] - skelcomp[i][0]) / lengths[i]
            dy = (skelcomp[i + 1][1] - skelcomp[i][1]) / lengths[i]
        if follow:
            mat = [[dx, -dy, x], [dy, dx, y]]
        else:
            mat = [[1, 0, x], [0, 1, y]]
        return Transform(mat)
    def center_node_at_origin(self, node):
        """Translates a node to the origin and applies translation if requested"""
        bbox = node.bounding_box()
        mat = Transform([[1, 0, -bbox.center.x], [0, 1, -bbox.center.y]])
        if self.options.vertical:
            bbox = BoundingBox(-bbox.y, -bbox.x)
            mat = Transform([[0, -1, 0], [1, 0, 0]]) * mat
        mat.add_translate([0, self.options.noffset])
        node.transform = mat * node.transform
        return bbox
    def effect(self):

        if len(self.svg.selection) < 2:
            inkex.errormsg(_("This extension requires two selected paths."))
            return
        original_pattern_node, skeletons = self.get_patterns_and_skeletons(False, False)

        g_node = Group()
        original_pattern_node.getparent().append(g_node)

        if self.options.copymode == "clone":
            pattern_node = g_node.add(Use())
            pattern_node.href = original_pattern_node
        else:
            pattern_node = original_pattern_node.duplicate()

        # We will later compute transforms relative to the origin
        bbox = self.center_node_at_origin(pattern_node)

        width = bbox.width
        dx = width + self.options.space
        if dx < 0.01:
            if isinstance(original_pattern_node, inkex.TextElement):
                raise inkex.AbortExtension("Please convert texts to path first")
            raise inkex.AbortExtension("The total length of the pattern is too small\n"\
                "Please choose a larger object or set 'Space between copies' > 0")

        # check if group and expand it
        pattern_list = []
        if self.options.grouppick and isinstance(pattern_node, Group):
            mat = pattern_node.transform
            for child in pattern_node:
                child.transform = mat * child.transform
                pattern_list.append(child)
        else:
            pattern_list.append(pattern_node)

        self._do_transform(skeletons, width, pattern_list, g_node)

        if self.options.copymode == "move":
            original_pattern_node.getparent().remove(original_pattern_node)
        # pattern_node was just a temporary copy, definitely remove this
        pattern_node.getparent().remove(pattern_node)

    def _do_transform(self, skeletons, width, pattern_list, g_node):
        counter = 0
        for skelnode in skeletons.values():
            skelnode.apply_transform()
            cur_skeleton = skelnode.path.to_superpath()
            for comp in cur_skeleton:
                skelcomp, lengths = self.linearize(comp)
                skel_closed = all([math.isclose(i, j) for i, j in zip(skelcomp[0], skelcomp[-1])])

                length = sum(lengths)
                dx = width + self.options.space
                if self.options.stretch:
                    n = int(length / dx + self.options.space)
                    if n > 0:
                        dx = (length) / n

                s = 0 if self.options.stretch else self.options.toffset * 0.01 * dx
                while s <= length:
                    local_transform = self.localTransformAt(s, skelcomp, lengths, skel_closed, \
                                                            self.options.follow)

                    pattern_idx = random.randint(0, len(pattern_list) - 1) if \
                                    self.options.pickmode == "rand" else counter % len(pattern_list)

                    clone = pattern_list[pattern_idx].copy()

                    g_node.append(clone)

                    clone.transform = local_transform * clone.transform
                    s += dx
                    counter += 1

if __name__ == '__main__':
    PathScatter().run()



# coding=utf-8
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=invalid-name
"""Deprecated cspsubdiv API"""

from inkex.deprecated import deprecate
from inkex import bezier

maxdist = deprecate(bezier.maxdist)
cspsubdiv = deprecate(bezier.cspsubdiv)
subdiv = deprecate(bezier.subdiv)



#!/usr/bin/env python
"""List all colors used in an svg document"""

from collections import defaultdict
import inkex

class ListColours(inkex.ColorExtension):
    """Make the colours darker"""
    _counts = defaultdict(int)

    def effect(self):
        super(ListColours, self).effect()
        items = sorted(self._counts.items(), key=lambda v: -v[1])
        for color, count in items:
            self.msg("{count}: {color}".format(color=color, count=count))

    def modify_color(self, name, color):
        self._counts[color] += 1
        return color

if __name__ == '__main__':
    ListColours().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (c) 2009 - Jos Hirth, kaioa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Export a gimp pallet file (.gpl)
"""

import inkex
from inkex import ShapeElement, ColorIdError, ColorError


class ExportGimpPalette(inkex.OutputExtension):
    """Export all colors in a document to a gimp pallet"""
    select_all = (ShapeElement,)
    names = {}

    def save(self, stream):
        name = self.svg.name.replace('.svg', '')
        stream.write('GIMP Palette\nName: {}\n#\n'.format(name).encode('utf-8'))

        for key, value in sorted(list(set(self.get_colors()))):
            stream.write("{} {}\n".format(key, value).encode('utf-8'))


    def get_colors(self):
        """Get all the colors from the selected elements"""
        for elem in self.svg.selection.filter(ShapeElement):
            for color in self.process_element(elem):
                if str(color).upper() == 'NONE':
                    continue
                yield ("{:3d} {:3d} {:3d}".format(*color.to_rgb()), self.names.get(color) or str(color).upper())

    def process_element(self, elem):
        """Recursively process elements for colors"""
        style = elem.fallback_style(move=False)
        for col in inkex.Style.color_props:
            try:
                col = inkex.Color(style.get(col))
                if (elem.getparent().get('inkscape:swatch') == "solid"):
                    self.names[col] = elem.getparent().get_id()
                yield col
            except ColorIdError:
                gradient = self.svg.getElementById(style.get(col))
                for stop in gradient.stops:
                    for item in self.process_element(stop):
                        yield item
            except ColorError:
                pass # Bad color

        if elem.href is not None: # Capture colors of symbols or clones pointing to defs
            for color in self.process_element(elem.href):
                yield color


if __name__ == '__main__':
    ExportGimpPalette().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2013 <Public Domain>
#               2018 Martin Owens <doctormo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import inkex
from inkex.ports import Serial
from inkex.localization import inkex_gettext as _

import hpgl_encoder

class Plot(inkex.EffectExtension):
    """Generate a plot in HPGL output"""
    def add_arguments(self, pars):
        pars.add_argument('--tab')
        pars.add_argument('--parallelPort', default='/dev/usb/lp2', help='Parallel port')
        pars.add_argument('--serialPort', default='COM1', help='Serial port')
        pars.add_argument('--serialBaudRate', default='9600', help='Serial Baud rate')
        pars.add_argument('--serialByteSize', default='eight', help='Serial byte size')
        pars.add_argument('--serialStopBits', default='one', help='Serial stop bits')
        pars.add_argument('--serialParity', default='none', help='Serial parity')
        pars.add_argument('--serialFlowControl', default='0', help='Flow control')
        pars.add_argument('--resolutionX', type=float, default=1016.0, help='Resolution X (dpi)')
        pars.add_argument('--resolutionY', type=float, default=1016.0, help='Resolution Y (dpi)')
        pars.add_argument('--pen', type=int, default=1, help='Pen number')
        pars.add_argument('--force', type=int, default=24, help='Pen force (g)')
        pars.add_argument('--speed', type=int, default=20, help='Pen speed (cm/s)')
        pars.add_argument('--orientation', default='90', help='Rotation (Clockwise)')
        pars.add_argument('--mirrorX', type=inkex.Boolean, default=False, help='Mirror X axis')
        pars.add_argument('--mirrorY', type=inkex.Boolean, default=False, help='Mirror Y axis')
        pars.add_argument('--center', type=inkex.Boolean, default=False, help='Center zero point')
        pars.add_argument('--overcut', type=float, default=1.0, help='Overcut (mm)')
        pars.add_argument('--precut', type=inkex.Boolean, default=True, help='Use precut')
        pars.add_argument('--flat', type=float, default=1.2, help='Curve flatness')
        pars.add_argument('--autoAlign', type=inkex.Boolean, default=True, help='Auto align')
        pars.add_argument('--toolOffset', type=float, default=0.25,\
            help='Tool (Knife) offset correction (mm)')
        pars.add_argument('--portType', type=self.arg_method('to'),\
            default=self.to_serial, dest="to_port", help='Port type')
        pars.add_argument('--commandLanguage', type=self.arg_method('convert'),\
            default=self.convert_hpgl, dest="to_language", help='Command Language Filter')

    def effect(self):
        # get hpgl data
        encoder = hpgl_encoder.hpglEncoder(self)
        try:
            self.options.to_port(self.options.to_language(encoder.getHpgl()))
        except hpgl_encoder.NoPathError:
            raise inkex.AbortExtension(_("No paths where found. Please convert objects to paths."))

    def convert_hpgl(self, hpgl):
        """Convert raw HPGL to HPGL"""
        init = 'IN'
       # if self.options.force > 0:
        #    init += ';FS%d' % self.options.force
       # if self.options.speed > 0:
        #    init += ';VS%d' % self.options.speed
        return init + hpgl + ';PU0,0;SP0;IN; '

    def convert_dmpl(self, hpgl):
        """Convert HPGL to DMPL"""
        # ;: = Initialise plotter
        # H = Home position
        # A = Absolute pen positioning
        # Ln = Line type
        # Pn = Pen select
        # Vn = velocity
        # ECn = Coordinate addressing, 1: 0.001 inch, 5: 0.005 inch, M: 0.1 mm
        # D = Pen down
        # U = Pen up
        # Z = Reset plotter
        # n,n, = Coordinate pair
        hpgl = hpgl.replace(';', ',')
        hpgl = hpgl.replace('SP', 'P')
        hpgl = hpgl.replace('PU', 'U')
        hpgl = hpgl.replace('PD', 'D')
        init = ';:HAL0'
        if self.options.speed > 0:
            init += 'V%d' % self.options.speed
        init += 'EC1'
        return init + hpgl[1:] + ',P0,U0,0,Z '

    def convert_knk(self, hpgl):
        """Convert HPGL to KNK Plotter Language"""
        init = 'ZG'
       # if self.options.force > 0:
        #    init += ';FS%d' % self.options.force
       # if self.options.speed > 0:
        #    init += ';VS%d' % self.options.speed
        return init + hpgl + ';SP0;PU0,0;@ '

    def to_parallel(self, hpgl):
        """Output to hgpl to a parallel port"""
        port = open(self.options.parallelPort, "wb")
        port.write(hpgl.encode('utf8'))
        port.close()

    def to_serial(self, hpgl):
        """Output to hgpl to a serial port"""
        with Serial(self.options.serialPort,
                    baud=self.options.serialBaudRate,
                    stop=self.options.serialStopBits,
                    size=self.options.serialByteSize,
                    flow=self.options.serialFlowControl,
                    parity=self.options.serialParity,
                   ) as comx:
            comx.write(hpgl.encode('utf8'))

if __name__ == '__main__':
    Plot().run()


#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2006 Jean-Francois Barraud, barraud@math.univ-lille1.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Distorts selected paths, stretching it vertically while squeezing horizontally.

Amount is controlled by ratio parameter.

Curve gives further effect by bunching the surface towards the ends.
"""

from inkex.utils import X, Y
from pathmodifier import Diffeo
import inkex


class RubberStretch(Diffeo):
    """Distort selected paths"""
    ratio = property(lambda self: -(self.options.ratio / 100))
    curve = property(lambda self: min(self.options.curve / 100, 0.99))

    def add_arguments(self, pars):
        pars.add_argument("-r", "--ratio", type=float, default=25)
        pars.add_argument("-c", "--curve", type=float, default=25)

    def applyDiffeo(self, bpt, vects=()):
        for vect in vects:
            vect[0] -= bpt[0]
            vect[1] -= bpt[1]
            vect[1] *= -1
        bpt[1] *= -1
        bx0 = self.bbox.center.x
        by0 = -self.bbox.center.y

        x, y = (bpt[0] - bx0), (bpt[1] - by0)
        sx1 = (1 + self.curve * (x / (self.bbox.width / 2) + 1) * \
              (x / (self.bbox.width / 2) - 1)) * 2 ** self.ratio
        sy1 = (1 + self.curve * (y / (self.bbox.height / 2) + 1) * \
              (y / (self.bbox.height / 2) - 1)) * 2 ** self.ratio
        bpt[0] = bx0 + x * sy1
        bpt[1] = by0 + y / sx1
        for vect in vects:
            dx_dx = sy1
            dx_dy = x * 2 * self.curve * y / self.bbox.height / self.bbox.height * 2 ** self.ratio
            dy_dx = -y * 2 * self.curve * x / self.bbox.width / \
                        self.bbox.width * 2 ** self.ratio / sx1 / sx1
            dy_dy = 1 / sx1
            vect[0] = dx_dx * vect[X] + dx_dy * vect[Y]
            vect[1] = dy_dx * vect[X] + dy_dy * vect[Y]

        # --spherify
        # s=((x*x+y*y)/(w*w+h*h))**(-a/2)
        # bpt[0]=x0+s*x
        # bpt[1]=y0+s*y
        # for v in vects:
        #    dx,dy=v
        #    v[0]=(1-a/2/(x*x+y*y)*2*x*x)*s*dx+( -a/2/(x*x+y*y)*2*y*x)*s*dy
        #    v[1]=( -a/2/(x*x+y*y)*2*x*y)*s*dx+(1-a/2/(x*x+y*y)*2*y*y)*s*dy

        for vect in vects:
            vect[0] += bpt[0]
            vect[1] += bpt[1]
            vect[1] *= -1
        bpt[1] *= -1


if __name__ == "__main__":
    RubberStretch().run()



#!/usr/bin/env python3
# coding=utf-8
#
# Copyright (C) 2012 Martin Owens
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
"""
This is a test framework setup.py only, not used for packaging.
"""

from setuptools import setup

setup(
        name='inkscape-core-extensions',
        version='0.0',
        description='Inkscape core extensions for testing',
        long_description='N/A',
        author='Inkscape Authors',
        url='https://gitlab.com/inkscape/extensions',
        author_email='developers@inkscape.org',
        test_suite='tests',
        platforms='linux',
        license='GPLv2',
        classifiers=[
            'Development Status :: 0 - Test Only',
            'Intended Audience :: Developers',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
        ],
        install_requires=['scour', 'numpy', 'pyserial'],
        setup_requires=["pytest-runner"],
        tests_require=["pytest", "pytest-cov"]
)



# -*- coding: utf-8 -*-
#
# Copyright (c) Aaron Spike <aaron@ekips.org>
#               Aurélio A. Heckert <aurium(a)gmail.com>
#               Bulia Byak <buliabyak@users.sf.net>
#               Nicolas Dufour, nicoduf@yahoo.fr
#               Peter J. R. Moulder <pjrm@users.sourceforge.net>
#               Martin Owens <doctormo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Convert to and from various units and find the closest matching unit.
"""

import re

# a dictionary of unit to user unit conversion factors
CONVERSIONS = {
    'in': 96.0,
    'pt': 1.3333333333333333,
    'px': 1.0,
    'mm': 3.779527559055118,
    'cm': 37.79527559055118,
    'm': 3779.527559055118,
    'km': 3779527.559055118,
    'Q': 0.94488188976378,
    'pc': 16.0,
    'yd': 3456.0,
    'ft': 1152.0,
    '': 1.0,  # Default px
}

# allowed unit types, including percentages, relative units, and others
# that are not suitable for direct conversion to a length.
# Note that this is _not_ an exhaustive list of allowed unit types.
UNITS = ['in', 'pt', 'px', 'mm', 'cm', 'm', 'km', 'Q', 'pc', 'yd', 'ft', '',\
    '%', 'em', 'ex', 'ch', 'rem', 'vw', 'vh', 'vmin', 'vmax',\
    'deg', 'grad', 'rad', 'turn', 's', 'ms', 'Hz', 'kHz',\
    'dpi', 'dpcm', 'dppx']

UNIT_MATCH = re.compile(r'({})'.format('|'.join(UNITS)))
NUMBER_MATCH = re.compile(r'(([-+]?[0-9]+(\.[0-9]*)?|[-+]?\.[0-9]+)([eE][-+]?[0-9]+)?)')
BOTH_MATCH = re.compile(r'^\s*{}\s*{}\s*$'.format(NUMBER_MATCH.pattern, UNIT_MATCH.pattern))


def parse_unit(value, default_unit='px', default_value=None):
    """
    Takes a value such as 55.32px and returns (55.32, 'px')
    Returns default (None) if no match can be found
    """
    ret = BOTH_MATCH.match(str(value))
    if ret:
        return float(ret.groups()[0]), ret.groups()[-1] or default_unit
    return (default_value, default_unit) if default_value is not None else None


def are_near_relative(point_a, point_b, eps=0.01):
    """Return true if the points are near to eps"""
    return (point_a - point_b <= point_a * eps) and (point_a - point_b >= -point_a * eps)


def discover_unit(value, viewbox, default='px'):
    """Attempt to detect the unit being used based on the viewbox"""
    # Default 100px when width can't be parsed
    (value, unit) = parse_unit(value, default_value=100.0)
    if unit not in CONVERSIONS:
        return default
    this_factor = CONVERSIONS[unit] * value / viewbox

    # try to find the svgunitfactor in the list of units known. If we don't find something, ...
    for unit, unit_factor in CONVERSIONS.items():
        if unit != '':
            # allow 1% error in factor
            if are_near_relative(this_factor, unit_factor, eps=0.01):
                return unit
    return default


def convert_unit(value, to_unit, default='px'):
    """Returns userunits given a string representation of units in another system"""
    value, from_unit = parse_unit(value, default_unit=default, default_value=0.0)
    if from_unit in CONVERSIONS and to_unit in CONVERSIONS:
        return value * CONVERSIONS[from_unit] / CONVERSIONS.get(to_unit, CONVERSIONS['px'])
    return 0.0


def render_unit(value, unit):
    """Checks and then renders a number with its unit"""
    try:
        if isinstance(value, str):
            (value, unit) = parse_unit(value, default_unit=unit)
        return f"{value:.6g}{ unit:s}"
    except TypeError:
        return ''



# coding=utf-8
#
# Copyright (C) 2010 Nick Drobchenko, nick@cnc-club.ru
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Basic common utility functions for calculated things
"""
import os
import sys
import random
import re
import math

from itertools import tee
from argparse import ArgumentTypeError

# All the names that get added to the inkex API itself.
__all__ = ('AbortExtension', 'DependencyError', 'Boolean', 'errormsg')

ABORT_STATUS = -5

(X, Y) = range(2)
PY3 = sys.version_info[0] == 3

# Taken from https://www.w3.org/Graphics/SVG/1.1/paths.html#PathDataBNF
DIGIT_REX_PART = r'[0-9]'
DIGIT_SEQUENCE_REX_PART = fr'(?:{DIGIT_REX_PART}+)'
INTEGER_CONSTANT_REX_PART = DIGIT_SEQUENCE_REX_PART
SIGN_REX_PART = r'[+-]'
EXPONENT_REX_PART = fr'(?:[eE]{SIGN_REX_PART}?{DIGIT_SEQUENCE_REX_PART})'
FRACTIONAL_CONSTANT_REX_PART = fr'(?:{DIGIT_SEQUENCE_REX_PART}?\.{DIGIT_SEQUENCE_REX_PART}|{DIGIT_SEQUENCE_REX_PART}\.)'
FLOATING_POINT_CONSTANT_REX_PART = fr'(?:{FRACTIONAL_CONSTANT_REX_PART}{EXPONENT_REX_PART}?|{DIGIT_SEQUENCE_REX_PART}{EXPONENT_REX_PART})'
NUMBER_REX = re.compile(fr'(?:{SIGN_REX_PART}?{FLOATING_POINT_CONSTANT_REX_PART}|{SIGN_REX_PART}?{INTEGER_CONSTANT_REX_PART})')

def _pythonpath():
    for pth in os.environ.get('PYTHONPATH', '').split(':'):
        if os.path.isdir(pth):
            yield pth

def get_user_directory():
    """Return the user directory where extensions are stored."""
    if 'INKSCAPE_PROFILE_DIR' in os.environ:
        return os.path.abspath(
            os.path.expanduser(
                os.path.join(os.environ['INKSCAPE_PROFILE_DIR'], 'extensions')))

    home = os.path.expanduser("~")
    for pth in _pythonpath():
        if pth.startswith(home):
            return pth

def get_inkscape_directory():
    """Return the system directory where inkscape's core is."""
    for pth in _pythonpath():
        if os.path.isdir(os.path.join(pth, 'inkex')):
            return pth


class KeyDict(dict):
    """
    A normal dictionary, except asking for anything not in the dictionary
    always returns the key itself. This is used for translation dictionaries.
    """
    def __getitem__(self, key):
        try:
            return super().__getitem__(key)
        except KeyError:
            return key


def Boolean(value):
    """ArgParser function to turn a boolean string into a python boolean"""
    if value.upper() == 'TRUE':
        return True
    elif value.upper() == 'FALSE':
        return False
    return None

def to_bytes(content):
    """Ensures the content is bytes"""
    if isinstance(content, bytes):
        return content
    return str(content).encode("utf8")

def debug(what):
    """Print debug message if debugging is switched on"""
    errormsg(what)
    return what

def do_nothing(*args, **kwargs): # pylint: disable=unused-argument
    """A blank function to do nothing"""
    pass

def errormsg(msg):
    """Intended for end-user-visible error messages.

       (Currently just writes to stderr with an appended newline, but could do
       something better in future: e.g. could add markup to distinguish error
       messages from status messages or debugging output.)

       Note that this should always be combined with translation:

         import inkex
         ...
         inkex.errormsg(_("This extension requires two selected paths."))
    """
    try:
        sys.stderr.write(msg)
    except TypeError:
        sys.stderr.write(str(msg))
    except UnicodeEncodeError:
        # Python 2:
        # Fallback for cases where sys.stderr.encoding is not Unicode.
        # Python 3:
        # This will not work as write() does not accept byte strings, but AFAIK
        # we should never reach this point as the default error handler is
        # 'backslashreplace'.

        # This will be None by default if stderr is piped, so use ASCII as a
        # last resort.
        encoding = sys.stderr.encoding or 'ascii'
        sys.stderr.write(msg.encode(encoding, 'backslashreplace'))

    # Write '\n' separately to avoid dealing with different string types.
    sys.stderr.write('\n')


class AbortExtension(Exception):
    """Raised to print a message to the user without backtrace"""

    def __init__(self, message=""):
        self.message = message

    def write(self):
        """write the error message out to the user"""
        errormsg(self.message)


class DependencyError(NotImplementedError):
    """Raised when we need an external python module that isn't available"""

class FragmentError(Exception):
    """Raised when trying to do rooty things on an xml fragment"""

def to(kind):  # pylint: disable=invalid-name
    """
    Decorator which will turn a generator into a list, tuple or other object type.
    """

    def _inner(call):
        def _outer(*args, **kw):
            return kind(call(*args, **kw))

        return _outer

    return _inner


def strargs(string, kind=float):
    """Returns a list of floats from a string with commas or space separators, 
        also splits at -(minus) signs by adding a space in front of the - sign
    """
    return [kind(val) for val in NUMBER_REX.findall(string)]


class classproperty:  # pylint: disable=invalid-name, too-few-public-methods
    """Combine classmethod and property decorators"""

    def __init__(self, func):
        self.func = func

    def __get__(self, obj, owner):
        return self.func(owner)


def filename_arg(name):
    """Existing file to read or option used in script arguments"""
    filename = os.path.abspath(os.path.expanduser(name))
    if not os.path.isfile(filename):
        raise ArgumentTypeError("File not found: {}".format(name))
    return filename

def pairwise(iterable, start=True):
    "Iterate over a list with overlapping pairs (see itertools recipes)"
    first, then = tee(iterable)
    starter = [(None, next(then, None))]
    if not start:
        starter = []
    return starter + list(zip(first, then))

EVAL_GLOBALS = {}
EVAL_GLOBALS.update(random.__dict__)
EVAL_GLOBALS.update(math.__dict__)

def math_eval(function, variable="x"):
    """Interpret a function string. All functions from math and random may be used.
    @returns a lambda expression if sucessful; otherwise None.
    """
    try:
        if function != "":
            return eval(f'lambda {variable}: ' + (function.strip('"') or 't'), EVAL_GLOBALS, {})
    # handle incomplete/invalid function gracefully
    except SyntaxError:
        pass
    return None



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2014 Martin Owens, email@doctormo.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
An extension to export multiple svg files from a single svg file containing layers.

Each defs is duplicated for each svg outputted.
"""

from __future__ import unicode_literals
import os
import sys
import copy
import tarfile
import io
import calendar
import time
import inkex

class TarLayers(inkex.OutputExtension):
    """Entry point to our layers export"""
    def make_template(self):
        """Returns the current document as a new empty document with the same defs"""
        newdoc = copy.deepcopy(self.document)
        for (name, layer) in self.layers(newdoc):
            layer.getparent().remove(layer)
        return newdoc

    def layers(self, document):
        for node in document.getroot().iterchildren():
            if isinstance(node, inkex.Layer) and node.label:
                yield (node.label, node)

    def io_document(self, name, doc):
        string = io.BytesIO()
        doc.write(string)
        info = tarfile.TarInfo(name=name+'.svg')
        info.mtime = calendar.timegm(time.gmtime())
        info.size  = string.tell()
        string.seek(0)
        return dict(tarinfo=info, fileobj=string)

    def save(self, stream):
        """Save the tar file output"""
        tar = tarfile.open(fileobj=stream, mode='w|')

        # Switch stdout to binary on Windows.
        if sys.platform == "win32":
            import msvcrt
            msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)

        template = self.make_template()

        previous = None
        for (name, _layer) in self.layers(self.document):
            layer = copy.deepcopy(_layer)
            if previous != None:
                template.getroot().replace(previous, layer)
            else:
                template.getroot().append(layer)
            previous = layer

            tar.addfile(**self.io_document(name, template))


if __name__ == '__main__':   #pragma: no cover
    TarLayers().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2008 Aurelio A. Heckert <aurium(a)gmail.com>
#    Week number option added by Olav Vitters and Nicolas Dufour (2012)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
A calendar generator plugin for Inkscape, but also can be used as a standalone
command line application.
#
More on ISO week number calculation on:
http://en.wikipedia.org/wiki/ISO_week_date
(The first week of a year is the week that contains the first Thursdayof the year.)
"""

__version__ = "0.3"

import datetime
import calendar
import re
import sys

import inkex
from inkex import TextElement

if sys.version_info[0] > 2:
    def unicode(s, encoding):
        if isinstance(s, bytes):
            return s.decode(encoding)
        return s


class Calendar(inkex.EffectExtension):
    """Generate Calendar in SVG"""
    def add_arguments(self, pars):
        pars.add_argument("--tab", type=str, dest="tab")
        pars.add_argument("--month", type=int, default=0,\
            help="Month to be generated. If 0, then the entry year will be generated.")
        pars.add_argument("--year", type=int, default=0,\
            help="Year to be generated. If 0, then the current year will be generated.")
        pars.add_argument("--fill-empty-day-boxes", type=inkex.Boolean,\
            dest="fill_edb", default=True, help="Fill empty day boxes with next month days.")
        pars.add_argument("--show-week-number", type=inkex.Boolean,\
            dest="show_weeknr", default=False, help="Include a week number column.")
        pars.add_argument("--start-day", default="sun", help='Week start day. ("sun" or "mon")')
        pars.add_argument("--weekend", default="sat+sun",\
            help='Define the weekend days. ("sat+sun" or "sat" or "sun")')
        pars.add_argument(
            "--auto-organize", type=inkex.Boolean, dest="auto_organize", default=True,
            help='Automatically set the size and positions.')
        pars.add_argument(
            "--months-per-line", type=int, dest="months_per_line", default=3,
            help='Number of months side by side.')
        pars.add_argument(
            "--month-width", type=str, dest="month_width", default="6cm",
            help='The width of the month days box.')
        pars.add_argument(
            "--month-margin", type=str, dest="month_margin", default="1cm",
            help='The space between the month boxes.')
        pars.add_argument(
            "--color-year", type=str, dest="color_year", default="#888",
            help='Color for the year header.')
        pars.add_argument(
            "--color-month", type=str, dest="color_month", default="#666",
            help='Color for the month name header.')
        pars.add_argument(
            "--color-day-name", type=str, dest="color_day_name", default="#999",
            help='Color for the week day names header.')
        pars.add_argument(
            "--color-day", type=str, dest="color_day", default="#000",
            help='Color for the common day box.')
        pars.add_argument(
            "--color-weekend", type=str, dest="color_weekend", default="#777",
            help='Color for the weekend days.')
        pars.add_argument(
            "--color-nmd", type=str, dest="color_nmd", default="#BBB",
            help='Color for the next month day, in empty day boxes.')
        pars.add_argument(
            "--color-weeknr", type=str, dest="color_weeknr", default="#808080",
            help='Color for the week numbers.')
        pars.add_argument(
            "--font-year", type=str, dest="font_year", default="arial",
            help='Font for the year string.')
        pars.add_argument(
            "--font-month", type=str, dest="font_month", default="arial",
            help='Font for the month strings.')
        pars.add_argument(
            "--font-day-name", type=str, dest="font_day_name", default="arial",
            help='Font for the days of the week strings.')
        pars.add_argument(
            "--font-day", type=str, dest="font_day", default="arial",
            help='Font for the day strings.')
        pars.add_argument(
            "--month-names", type=str, dest="month_names",
            default='January February March '
                    'April May June July '
                    'August September October '
                    'November December',
            help='The month names for localization.')
        pars.add_argument(
            "--day-names", type=str, dest="day_names", default='Sun Mon Tue Wed Thu Fri Sat',
            help='The week day names for localization.')
        pars.add_argument(
            "--weeknr-name", type=str, dest="weeknr_name", default='Wk',
            help='The week number column name for localization.')
        pars.add_argument(
            "--encoding", type=str, dest="input_encode", default='utf-8',
            help='The input encoding of the names.')

    def validate_options(self):
        # inkex.errormsg( self.options.input_encode )
        # Convert string names lists in real lists
        m = re.match(r'\s*(.*[^\s])\s*', self.options.month_names)
        self.options.month_names = re.split(r'\s+', m.group(1))
        m = re.match(r'\s*(.*[^\s])\s*', self.options.day_names)
        self.options.day_names = re.split(r'\s+', m.group(1))
        # Validate names lists
        if len(self.options.month_names) != 12:
            inkex.errormsg('The month name list "' +
                           str(self.options.month_names) +
                           '" is invalid. Using default.')
            self.options.month_names = ['January', 'February', 'March',
                                        'April', 'May', 'June',
                                        'July', 'August', 'September',
                                        'October', 'November', 'December']
        if len(self.options.day_names) != 7:
            inkex.errormsg('The day name list "' +
                           str(self.options.day_names) +
                           '" is invalid. Using default.')
            self.options.day_names = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu',
                                      'Fri', 'Sat']
        # Convert year 0 to current year
        if self.options.year == 0:
            self.options.year = datetime.datetime.today().year
        # Year 1 starts it's week at monday, obligatorily
        if self.options.year == 1:
            self.options.start_day = 'mon'
        # Set the calendar start day
        if self.options.start_day == 'sun':
            calendar.setfirstweekday(6)
        else:
            calendar.setfirstweekday(0)
        # Convert string numbers with unit to user space float numbers
        self.options.month_width = self.svg.unittouu(self.options.month_width)
        self.options.month_margin = self.svg.unittouu(self.options.month_margin)

    # initial values
    month_x_pos = 0
    month_y_pos = 0
    weeknr = 0

    def calculate_size_and_positions(self):
        # month_margin month_width months_per_line auto_organize
        self.doc_w = self.svg.unittouu(self.document.getroot().get('width'))
        self.doc_h = self.svg.unittouu(self.document.getroot().get('height'))
        if self.options.show_weeknr:
            self.cols_before = 1
        else:
            self.cols_before = 0
        if self.options.auto_organize:
            if self.doc_h > self.doc_w:
                self.months_per_line = 3
            else:
                self.months_per_line = 4
        else:
            self.months_per_line = self.options.months_per_line
        # self.month_w = self.doc_w / self.months_per_line
        if self.options.auto_organize:
            self.month_w = (self.doc_w * 0.8) / self.months_per_line
            self.month_margin = self.month_w / 10
        else:
            self.month_w = self.options.month_width
            self.month_margin = self.options.month_margin
        self.day_w = self.month_w / (7 + self.cols_before)
        self.day_h = self.month_w / 9
        self.month_h = self.day_w * 7
        if self.options.month == 0:
            self.year_margin = ((self.doc_w + self.day_w -
                                 (self.month_w * self.months_per_line) -
                                 (self.month_margin *
                                  (self.months_per_line - 1))) / 2)  # - self.month_margin
        else:
            self.year_margin = (self.doc_w - self.month_w) / 2
        self.style_day = {
            'font-size': str(self.day_w / 2),
            'font-family': self.options.font_day,
            'text-anchor': 'middle',
            'text-align': 'center',
            'fill': self.options.color_day
        }
        self.style_weekend = self.style_day.copy()
        self.style_weekend['fill'] = self.options.color_weekend
        self.style_nmd = self.style_day.copy()
        self.style_nmd['fill'] = self.options.color_nmd
        self.style_month = self.style_day.copy()
        self.style_month['fill'] = self.options.color_month
        self.style_month['font-family'] = self.options.font_month
        self.style_month['font-size'] = str(self.day_w / 1.5)
        self.style_month['font-weight'] = 'bold'
        self.style_day_name = self.style_day.copy()
        self.style_day_name['fill'] = self.options.color_day_name
        self.style_day_name['font-family'] = self.options.font_day_name
        self.style_day_name['font-size'] = str(self.day_w / 3)
        self.style_year = self.style_day.copy()
        self.style_year['fill'] = self.options.color_year
        self.style_year['font-family'] = self.options.font_year
        self.style_year['font-size'] = str(self.day_w * 2)
        self.style_year['font-weight'] = 'bold'
        self.style_weeknr = self.style_day.copy()
        self.style_weeknr['fill'] = self.options.color_weeknr
        self.style_weeknr['font-size'] = str(self.day_w / 3)

    def is_weekend(self, pos):
        # weekend values: "sat+sun" or "sat" or "sun"
        if self.options.start_day == 'sun':
            if self.options.weekend == 'sat+sun' and pos == 0:
                return True
            if self.options.weekend == 'sat+sun' and pos == 6:
                return True
            if self.options.weekend == 'sat' and pos == 6:
                return True
            if self.options.weekend == 'sun' and pos == 0:
                return True
        else:
            if self.options.weekend == 'sat+sun' and pos == 5:
                return True
            if self.options.weekend == 'sat+sun' and pos == 6:
                return True
            if self.options.weekend == 'sat' and pos == 5:
                return True
            if self.options.weekend == 'sun' and pos == 6:
                return True
        return False

    def in_line_month(self, cal):
        cal2 = []
        for week in cal:
            for day in week:
                if day != 0:
                    cal2.append(day)
        return cal2

    def write_month_header(self, g, m):
        txt_atts = {'style': str(inkex.Style(self.style_month)),
                    'x': str((self.month_w - self.day_w) / 2),
                    'y': str(self.day_h / 5)}
        try:
            g.add(TextElement(**txt_atts)).text = unicode(
                self.options.month_names[m - 1],
                self.options.input_encode)
        except:
            raise ValueError('You must select a correct system encoding.')

        gw = g.add(inkex.Group())
        week_x = 0
        if self.options.start_day == 'sun':
            day_names = self.options.day_names[:]
        else:
            day_names = self.options.day_names[1:]
            day_names.append(self.options.day_names[0])

        if self.options.show_weeknr:
            day_names.insert(0, self.options.weeknr_name)

        for wday in day_names:
            txt_atts = {'style': str(inkex.Style(self.style_day_name)),
                        'x': str(self.day_w * week_x),
                        'y': str(self.day_h)}
            try:
                gw.add(TextElement(**txt_atts)).text = unicode(
                    wday, self.options.input_encode)
            except:
                raise ValueError('You must select a correct system encoding.')

            week_x += 1

    def create_month(self, m):
        txt_atts = {
            'transform': 'translate(' +
                         str(self.year_margin +
                             (self.month_w + self.month_margin) *
                             self.month_x_pos) +
                         ',' +
                         str((self.day_h * 4) +
                             (self.month_h * self.month_y_pos)) +
                         ')',
            'id': 'month_' +
                  str(m) +
                  '_' +
                  str(self.options.year)}
        g = self.year_g.add(inkex.Group(**txt_atts))
        self.write_month_header(g, m)
        gdays = g.add(inkex.Group())
        cal = calendar.monthcalendar(self.options.year, m)
        if m == 1:
            if self.options.year > 1:
                before_month = \
                    self.in_line_month(calendar.monthcalendar(self.options.year - 1, 12))
        else:
            before_month = \
                self.in_line_month(calendar.monthcalendar(self.options.year, m - 1))
        if m == 12:
            next_month = \
                self.in_line_month(calendar.monthcalendar(self.options.year + 1, 1))
        else:
            next_month = \
                self.in_line_month(calendar.monthcalendar(self.options.year, m + 1))
        if len(cal) < 6:
            # add a line after the last week
            cal.append([0, 0, 0, 0, 0, 0, 0])
        if len(cal) < 6:
            # add a line before the first week (Feb 2009)
            cal.reverse()
            cal.append([0, 0, 0, 0, 0, 0, 0])
            cal.reverse()
        # How mutch before month days will be showed:
        bmd = cal[0].count(0) + cal[1].count(0)
        before = True
        week_y = 0
        for week in cal:
            if (self.weeknr != 0 and
                ((self.options.start_day == 'mon' and week[0] != 0) or
                 (self.options.start_day == 'sun' and week[1] != 0))) or \
                    (self.weeknr == 0 and
                     ((self.options.start_day == 'mon' and week[3] > 0) or
                      (self.options.start_day == 'sun' and week[4] > 0))):
                self.weeknr += 1
            week_x = 0
            if self.options.show_weeknr:
                # Remove leap week (starting previous year) and empty weeks
                if self.weeknr != 0 and not (week[0] == 0 and week[6] == 0):
                    style = self.style_weeknr
                    txt_atts = {'style': str(inkex.Style(style)),
                                'x': str(self.day_w * week_x),
                                'y': str(self.day_h * (week_y + 2))}
                    gdays.add(TextElement(**txt_atts)).text = str(self.weeknr)
                    week_x += 1
                else:
                    week_x += 1
            for day in week:
                style = self.style_day
                if self.is_weekend(week_x - self.cols_before):
                    style = self.style_weekend
                if day == 0:
                    style = self.style_nmd
                txt_atts = {'style': str(inkex.Style(style)),
                            'x': str(self.day_w * week_x),
                            'y': str(self.day_h * (week_y + 2))}
                text = None
                if day == 0 and not self.options.fill_edb:
                    pass  # draw nothing
                elif day == 0:
                    if before:
                        text = str(before_month[-bmd])
                        bmd -= 1
                    else:
                        text = str(next_month[bmd])
                        bmd += 1
                else:
                    text = str(day)
                    before = False
                if text:
                    gdays.add(TextElement(**txt_atts)).text = text
                week_x += 1
            week_y += 1
        self.month_x_pos += 1
        if self.month_x_pos >= self.months_per_line:
            self.month_x_pos = 0
            self.month_y_pos += 1

    def effect(self):
        self.validate_options()
        self.calculate_size_and_positions()
        parent = self.document.getroot()
        txt_atts = {'id': 'year_' + str(self.options.year)}
        self.year_g = parent.add(inkex.Group(**txt_atts))
        txt_atts = {'style': str(inkex.Style(self.style_year)),
                    'x': str(self.doc_w / 2),
                    'y': str(self.day_w * 1.5)}
        self.year_g.add(TextElement(**txt_atts)).text = str(self.options.year)
        try:
            if self.options.month == 0:
                for m in range(1, 13):
                    self.create_month(m)
            else:
                self.create_month(self.options.month)
        except ValueError as err:
            return inkex.errormsg(str(err))

if __name__ == '__main__':
    Calendar().run()



#!/usr/bin/env python
#
# Copyright 2008, 2009 Hannes Hochreiner
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.
#
"""Effect to print out jessyInk summary"""

from collections import defaultdict
import inkex

from jessyink_install import JessyInkMixin, _

class Summary(JessyInkMixin, inkex.EffectExtension):
    """Print of jessyInk summary"""
    def add_arguments(self, pars):
        pars.add_argument('--tab')

    def effect(self):
        self.is_installed()

        # Find the script node, if present
        for node in self.svg.xpath("//svg:script[@id='JessyInk']"):
            version = node.get("jessyink:version")
            if version:
                self.msg(_(f"JessyInk script version {version} installed."))
            else:
                self.msg(_("JessyInk script installed."))

        slides = []
        master_slide = None

        for node in self.svg.descendants().get(inkex.Layer):
            if node.get("jessyink:master_slide"):
                master_slide = node
            else:
                slides.append(node)

        if master_slide is not None:
            self.msg(_("\nMaster slide:"))
            self.describe_node(master_slide, "\t",\
                ["<the number of the slide>", len(slides), "<the title of the slide>"])

        for i, slide in enumerate(slides):
            self.msg(_("\nSlide {0!s}:").format(i+1))
            self.describe_node(slide, "\t", [i + 1, len(slides), slide.label])

    def describe_node(self, node, prefix, dat):
        """Standard print out formatter"""

        self.msg(_(f"{prefix}Layer name: {node.label}"))
        self.describe_transition(node, prefix, "In")
        self.describe_transition(node, prefix, "Out")
        self.describe_autotext(node, prefix, dat)
        self.describe_effects(node, prefix)

    def describe_transition(self, node, prefix, transition):
        """Display information about transitions."""
        trans = inkex.Style(node.get(f"jessyink:transition{transition}"))
        if trans:
            name = trans["name"]
            if name != "appear" and "length" in trans:
                length = int(trans["length"] / 1000.0)
                self.msg(_(f"{prefix}Transition {transition}: {name} ({length!s} s)"))
            else:
                self.msg(_(f"{prefix}Transition {transition}: {name}"))

    def describe_autotext(self, node, prefix, dat):
        """Display information about auto-texts."""
        auto_texts = {"slide_num" : dat[0], "num" : dat[1], "title" : dat[2]}
        for x, child in enumerate(node.xpath(".//*[@jessyink:autoText]")):
            if not x:
                self.msg(_(f"\n{prefix}Auto-texts:"))

            pid = child.getparent().get("id")
            val = auto_texts[child.get('jessyink:autoText')]
            self.msg(_(
                f'{prefix}\t"{child.text}" (object id "{pid}") will be replaced by "{val}".'))

    def describe_effects(self, node, prefix):
        """Display information about effects."""
        effects = sorted(self.collect_effects(node), key=lambda val: val[0])
        for x, (enum, effect) in enumerate(effects):
            ret = ""

            order = effect[0]["order"]
            if not x:
                ret += _(f"\n{prefix}Initial effect (order number {order}):")
            else:
                ret += _(f"\n{prefix}Effect {enum!s} (order number {order}):")

            for item in effect:
                eid = item["id"]
                if item["type"] == "view":
                    ret += _(f"{prefix}\tView will be set according to object \"{eid}\"")
                else:
                    ret += _(f"{prefix}\tObject \"{eid}\"")

                    if item["direction"] == "in":
                        ret += _(" will appear")
                    elif item["direction"] == "out":
                        ret += _(" will disappear")

                if item["name"] != "appear":
                    ret += _(" using effect \"{0}\"").format(item["name"])

                if "length" in item:
                    ret += _(" in {0!s} s").format(int(item["length"]) / 1000.0)

            self.msg(ret + ".\n")

    @staticmethod
    def collect_effects(node):
        """Collect information about effects."""
        effects = defaultdict(list)
        for child in node.xpath(".//*[@jessyink:effectIn]"):
            effect_data = inkex.Style(child.get('jessyink:effectIn'))
            effect_data["direction"] = "in"
            effect_data["id"] = child.get("id")
            effect_data["type"] = "effect"
            effects[effect_data["order"]].append(effect_data)

        for child in node.xpath(".//*[@jessyink:effectOut]"):
            effect_data = inkex.Style(child.get('jessyink:effectOut'))
            effect_data["direction"] = "out"
            effect_data["id"] = child.get("id")
            effect_data["type"] = "effect"
            effects[effect_data["order"]].append(effect_data)

        for child in node.xpath(".//*[@jessyink:view]"):
            effect_data = inkex.Style(child.get('jessyink:view'))
            effect_data["id"] = child.get("id")
            effect_data["type"] = "view"
            effects[effect_data["order"]].append(effect_data)
        return effects

if __name__ == '__main__':
    Summary().run()



# coding=utf-8
#
# Copyright (C) 2006 Jos Hirth, kaioa.com
# Copyright (C) 2007 Aaron C. Spike
# Copyright (C) 2009 Monash University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Basic color controls
"""

from .tween import interpcoord

# All the names that get added to the inkex API itself.
__all__ = ('Color', 'ColorError', 'ColorIdError')

SVG_COLOR = {
    'aliceblue': '#f0f8ff',
    'antiquewhite': '#faebd7',
    'aqua': '#00ffff',
    'aquamarine': '#7fffd4',
    'azure': '#f0ffff',
    'beige': '#f5f5dc',
    'bisque': '#ffe4c4',
    'black': '#000000',
    'blanchedalmond': '#ffebcd',
    'blue': '#0000ff',
    'blueviolet': '#8a2be2',
    'brown': '#a52a2a',
    'burlywood': '#deb887',
    'cadetblue': '#5f9ea0',
    'chartreuse': '#7fff00',
    'chocolate': '#d2691e',
    'coral': '#ff7f50',
    'cornflowerblue': '#6495ed',
    'cornsilk': '#fff8dc',
    'crimson': '#dc143c',
    'cyan': '#00ffff',
    'darkblue': '#00008b',
    'darkcyan': '#008b8b',
    'darkgoldenrod': '#b8860b',
    'darkgray': '#a9a9a9',
    'darkgreen': '#006400',
    'darkgrey': '#a9a9a9',
    'darkkhaki': '#bdb76b',
    'darkmagenta': '#8b008b',
    'darkolivegreen': '#556b2f',
    'darkorange': '#ff8c00',
    'darkorchid': '#9932cc',
    'darkred': '#8b0000',
    'darksalmon': '#e9967a',
    'darkseagreen': '#8fbc8f',
    'darkslateblue': '#483d8b',
    'darkslategray': '#2f4f4f',
    'darkslategrey': '#2f4f4f',
    'darkturquoise': '#00ced1',
    'darkviolet': '#9400d3',
    'deeppink': '#ff1493',
    'deepskyblue': '#00bfff',
    'dimgray': '#696969',
    'dimgrey': '#696969',
    'dodgerblue': '#1e90ff',
    'firebrick': '#b22222',
    'floralwhite': '#fffaf0',
    'forestgreen': '#228b22',
    'fuchsia': '#ff00ff',
    'gainsboro': '#dcdcdc',
    'ghostwhite': '#f8f8ff',
    'gold': '#ffd700',
    'goldenrod': '#daa520',
    'gray': '#808080',
    'grey': '#808080',
    'green': '#008000',
    'greenyellow': '#adff2f',
    'honeydew': '#f0fff0',
    'hotpink': '#ff69b4',
    'indianred': '#cd5c5c',
    'indigo': '#4b0082',
    'ivory': '#fffff0',
    'khaki': '#f0e68c',
    'lavender': '#e6e6fa',
    'lavenderblush': '#fff0f5',
    'lawngreen': '#7cfc00',
    'lemonchiffon': '#fffacd',
    'lightblue': '#add8e6',
    'lightcoral': '#f08080',
    'lightcyan': '#e0ffff',
    'lightgoldenrodyellow': '#fafad2',
    'lightgray': '#d3d3d3',
    'lightgreen': '#90ee90',
    'lightgrey': '#d3d3d3',
    'lightpink': '#ffb6c1',
    'lightsalmon': '#ffa07a',
    'lightseagreen': '#20b2aa',
    'lightskyblue': '#87cefa',
    'lightslategray': '#778899',
    'lightslategrey': '#778899',
    'lightsteelblue': '#b0c4de',
    'lightyellow': '#ffffe0',
    'lime': '#00ff00',
    'limegreen': '#32cd32',
    'linen': '#faf0e6',
    'magenta': '#ff00ff',
    'maroon': '#800000',
    'mediumaquamarine': '#66cdaa',
    'mediumblue': '#0000cd',
    'mediumorchid': '#ba55d3',
    'mediumpurple': '#9370db',
    'mediumseagreen': '#3cb371',
    'mediumslateblue': '#7b68ee',
    'mediumspringgreen': '#00fa9a',
    'mediumturquoise': '#48d1cc',
    'mediumvioletred': '#c71585',
    'midnightblue': '#191970',
    'mintcream': '#f5fffa',
    'mistyrose': '#ffe4e1',
    'moccasin': '#ffe4b5',
    'navajowhite': '#ffdead',
    'navy': '#000080',
    'oldlace': '#fdf5e6',
    'olive': '#808000',
    'olivedrab': '#6b8e23',
    'orange': '#ffa500',
    'orangered': '#ff4500',
    'orchid': '#da70d6',
    'palegoldenrod': '#eee8aa',
    'palegreen': '#98fb98',
    'paleturquoise': '#afeeee',
    'palevioletred': '#db7093',
    'papayawhip': '#ffefd5',
    'peachpuff': '#ffdab9',
    'peru': '#cd853f',
    'pink': '#ffc0cb',
    'plum': '#dda0dd',
    'powderblue': '#b0e0e6',
    'purple': '#800080',
    'rebeccapurple': '#663399',
    'red': '#ff0000',
    'rosybrown': '#bc8f8f',
    'royalblue': '#4169e1',
    'saddlebrown': '#8b4513',
    'salmon': '#fa8072',
    'sandybrown': '#f4a460',
    'seagreen': '#2e8b57',
    'seashell': '#fff5ee',
    'sienna': '#a0522d',
    'silver': '#c0c0c0',
    'skyblue': '#87ceeb',
    'slateblue': '#6a5acd',
    'slategray': '#708090',
    'slategrey': '#708090',
    'snow': '#fffafa',
    'springgreen': '#00ff7f',
    'steelblue': '#4682b4',
    'tan': '#d2b48c',
    'teal': '#008080',
    'thistle': '#d8bfd8',
    'tomato': '#ff6347',
    'turquoise': '#40e0d0',
    'violet': '#ee82ee',
    'wheat': '#f5deb3',
    'white': '#ffffff',
    'whitesmoke': '#f5f5f5',
    'yellow': '#ffff00',
    'yellowgreen': '#9acd32',
    'none': None,
}
COLOR_SVG = dict([(value, name) for name, value in SVG_COLOR.items()])

def is_color(color):
    """Determine if it is a color that we can use. If not, leave it unchanged."""
    try:
        return bool(Color(color))
    except ColorError:
        return False

def constrain(minim, value, maxim, channel):
    """Returns the value so long as it is between min and max values"""
    if channel == 'h': # Hue
        return value % maxim # Wrap around hue value
    return min([maxim, max([minim, value])])

class ColorError(KeyError):
    """Specific color parsing error"""

class ColorIdError(ColorError):
    """Special color error for gradient and color stop ids"""

class Color(list):
    """An RGB array for the color"""
    red = property(lambda self: self.to_rgb()[0])
    red = red.setter(lambda self, value: self._set(0, value))
    green = property(lambda self: self.to_rgb()[1])
    green = green.setter(lambda self, value: self._set(1, value))
    blue = property(lambda self: self.to_rgb()[2])
    blue = blue.setter(lambda self, value: self._set(2, value))
    alpha = property(lambda self: self.to_rgba()[3])
    alpha = alpha.setter(lambda self, value: self._set(3, value, ('rgba',)))
    hue = property(lambda self: self.to_hsl()[0])
    hue = hue.setter(lambda self, value: self._set(0, value, ('hsl',)))
    saturation = property(lambda self: self.to_hsl()[1])
    saturation = saturation.setter(lambda self, value: self._set(1, value, ('hsl',)))
    lightness = property(lambda self: self.to_hsl()[2])
    lightness = lightness.setter(lambda self, value: self._set(2, value, ('hsl',)))

    def __init__(self, color=None, space='rgb'):
        super().__init__()
        if isinstance(color, Color):
            space, color = color.space, list(color)

        if isinstance(color, str):
            # String from xml or css attributes
            space, color = self.parse_str(color.strip())

        if isinstance(color, int):
            # Number from arg parser colour value
            space, color = self.parse_int(color)

        # Empty list means 'none', or no color
        if color is None:
            color = []

        if not isinstance(color, (list, tuple)):
            raise ColorError("Not a known a color value")

        self.space = space
        try:
            for val in color:
                self.append(val)
        except ValueError:
            raise ColorError("Bad color list")

    def __hash__(self):
        """Allow colors to be hashable"""
        return tuple(self.to_rgba()).__hash__()

    def _set(self, index, value, spaces=('rgb', 'rgba')):
        """Set the color value in place, limits setter to specific color space"""
        # Named colors are just rgb, so dump name memory
        if self.space == 'named':
            self.space = 'rgb'
        if not self.space in spaces:
            if index == 3 and self.space == 'rgb':
                # Special, add alpha, don't convert back to rgb
                self.space = 'rgba'
                self.append(constrain(0.0, float(value), 1.0, 'a'))
                return
            # Set in other colour space and convert back and forth
            target = self.to(spaces[0])
            target[index] = constrain(0, int(value), 255, spaces[0][index])
            self[:] = target.to(self.space)
            return
        self[index] = constrain(0, int(value), 255, spaces[0][index])

    def append(self, val):
        """Append a value to the local list"""
        if len(self) == len(self.space):
            raise ValueError("Can't add any more values to color.")

        if isinstance(val, str):
            val = val.strip()
            if val.endswith('%'):
                val = float(val.strip('%')) / 100
            else:
                val = float(val)

        end_type = int
        if len(self) == 3: # Alpha value
            val = min([1.0, val])
            end_type = float
        elif isinstance(val, float) and val <= 1.0:
            val *= 255

        if isinstance(val, (int, float)):
            super().append(max(end_type(val), 0))

    @staticmethod
    def parse_str(color):
        """Creates a rgb int array"""
        # Handle pre-defined svg color values
        if color and color.lower() in SVG_COLOR:
            return 'named', Color.parse_str(SVG_COLOR[color.lower()])[1]

        if color is None:
            return 'rgb', None

        if color.startswith('url('):
            raise ColorIdError("Color references other element id, e.g. a gradient")

        # Next handle short colors (css: #abc -> #aabbcc)
        if color.startswith('#'):
            # Remove any icc or ilab directives
            # FUTURE: We could use icc or ilab information
            col = color.split(' ')[0]
            if len(col) == 4:
                col = '#{1}{1}{2}{2}{3}{3}'.format(*col)

            # Convert hex to integers
            try:
                return 'rgb', (int(col[1:3], 16), int(col[3:5], 16), int(col[5:], 16))
            except ValueError:
                raise ColorError(f"Bad RGB hex color value {col}")

        # Handle other css color values
        elif '(' in color and ')' in color:
            space, values = color.lower().strip().strip(')').split('(')
            return space, values.split(',')

        try:
            return Color.parse_int(int(color))
        except ValueError:
            pass

        raise ColorError(f"Unknown color format: {color}")

    @staticmethod
    def parse_int(color):
        """Creates an rgb or rgba from a long int"""
        space = 'rgb'
        color = [
            ((color >> 24) & 255), # red
            ((color >> 16) & 255), # green
            ((color >> 8) & 255), # blue
            ((color & 255) / 255.), # opacity
        ]
        if color[-1] == 1.0:
            color.pop()
        else:
            space = 'rgba'
        return space, color

    def __str__(self):
        """int array to #rrggbb"""
        if not self:
            return 'none'
        if self.space == 'named':
            rgbhex = '#{0:02x}{1:02x}{2:02x}'.format(*self)
            if rgbhex in COLOR_SVG:
                return COLOR_SVG[rgbhex]
            self.space = 'rgb'
        if self.space == 'rgb':
            return '#{0:02x}{1:02x}{2:02x}'.format(*self)
        if self.space == 'rgba':
            if self[3] == 1.0:
                return 'rgb({:g}, {:g}, {:g})'.format(*self[:3])
            return 'rgba({:g}, {:g}, {:g}, {:g})'.format(*self)
        elif self.space == 'hsl':
            return 'hsl({0:g}, {1:g}, {2:g})'.format(*self)
        raise ColorError(f"Can't print colour space '{self.space}'")

    def __int__(self):
        """int array to large integer"""
        if not self:
            return -1
        color = self.to_rgba()
        return (color[0] << 24) + (color[1] << 16) + (color[2] << 8) + (int(color[3] * 255))

    def to(self, space):
        """Dynamic caller for to_hsl, to_rgb, etc"""
        return getattr(self, 'to_' + space)()

    def to_hsl(self):
        """Turn this color into a Hue/Saturation/Lightness colour space"""
        if not self and self.space in ('rgb', 'named'):
            return self.to_rgb().to_hsl()
        if self.space == 'hsl':
            return self
        elif self.space == 'rgb':
            return Color(rgb_to_hsl(*self.to_floats()), space='hsl')
        raise ColorError(f"Unknown color conversion {self.space}->hsl")

    def to_rgb(self):
        """Turn this color into a Red/Green/Blue colour space"""
        if not self and self.space in ('rgb', 'named'):
            return Color([0, 0, 0])
        if self.space == 'rgb':
            return self
        if self.space in ('rgba', 'named'):
            return Color(self[:3], space='rgb')
        elif self.space == 'hsl':
            return Color(hsl_to_rgb(*self.to_floats()), space='rgb')
        raise ColorError(f"Unknown color conversion {self.space}->rgb")

    def to_rgba(self, alpha=1.0):
        """Turn this color isn't an RGB with Alpha colour space"""
        if self.space == 'rgba':
            return self
        return Color(self.to_rgb() + [alpha], 'rgba')

    def to_floats(self):
        """Returns the colour values as percentage floats (0.0 - 1.0)"""
        return [val / 255.0 for val in self]

    def to_named(self):
        """Convert this color to a named color if possible"""
        if not self:
            return Color()
        return Color(COLOR_SVG.get(str(self), str(self)))

    def interpolate(self, other, fraction):
        """Iterpolate two colours by the given fraction"""
        return Color(
            [interpcoord(c1, c2, fraction)
             for (c1, c2) in zip(self.to_floats(), other.to_floats())]
            )


def rgb_to_hsl(red, green, blue):
    """RGB to HSL colour conversion"""
    rgb_max = max(red, green, blue)
    rgb_min = min(red, green, blue)
    delta = rgb_max - rgb_min
    hsl = [0.0, 0.0, (rgb_max + rgb_min) / 2.0]
    if delta != 0:
        if hsl[2] <= 0.5:
            hsl[1] = delta / (rgb_max + rgb_min)
        else:
            hsl[1] = delta / (2 - rgb_max - rgb_min)

        if red == rgb_max:
            hsl[0] = (green - blue) / delta
        elif green == rgb_max:
            hsl[0] = 2.0 + (blue - red) / delta
        elif blue == rgb_max:
            hsl[0] = 4.0 + (red - green) / delta

        hsl[0] /= 6.0
        if hsl[0] < 0:
            hsl[0] += 1
        if hsl[0] > 1:
            hsl[0] -= 1
    return hsl


def hsl_to_rgb(hue, sat, light):
    """HSL to RGB Color Conversion"""
    if sat == 0:
        return [light, light, light]  # Gray

    if light < 0.5:
        val2 = light * (1 + sat)
    else:
        val2 = light + sat - light * sat
    val1 = 2 * light - val2
    return [_hue_to_rgb(val1, val2, hue * 6 + 2.0),
            _hue_to_rgb(val1, val2, hue * 6),
            _hue_to_rgb(val1, val2, hue * 6 - 2.0)]


def _hue_to_rgb(val1, val2, hue):
    if hue < 0:
        hue += 6.0
    if hue > 6:
        hue -= 6.0
    if hue < 1:
        return val1 + (val2 - val1) * hue
    if hue < 3:
        return val2
    if hue < 4:
        return val1 + (val2 - val1) * (4 - hue)
    return val1



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2011 Felipe Correa da Silva Sanches <juca@members.fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""Convert known layer structures to svg font glyphs"""

import inkex
from inkex import SVGfont, FontFace, Glyph

class LayersToSvgFont(inkex.EffectExtension):
    """Convert layers to an svg font"""
    def guideline_value(self, label, index):
        for guide in self.svg.namedview.get_guides():
            if guide.label == label:
                return guide.point[index]
        return 0

    def flip_cordinate_system(self, path, emsize, baseline):
        """Flip Y axis because SVG fonts use a different orientation than regular SVG"""
        path = path.copy()
        path.transform.add_scale(1, -1)
        path.transform.add_translate(0, int(emsize) - int(baseline))
        path.apply_transform()
        return str(path.path)

    def effect(self):
        """Main extension code"""
        emsize = int(float(self.svg.get("width")))
        baseline = self.guideline_value("baseline", 1)
        ascender = self.guideline_value("ascender", 1) - baseline
        caps = self.guideline_value("caps", 1) - baseline
        xheight = self.guideline_value("xheight", 1) - baseline
        descender = baseline - self.guideline_value("descender", 1)

        font = self.svg.defs.get_or_create('svg:font', SVGfont)
        font.set("horiz-adv-x", str(emsize))
        font.set("horiz-origin-y", str(baseline))

        fontface = font.get_or_create('font-face', FontFace)
        fontface.set("font-family", "SVGFont")
        fontface.set("units-per-em", str(emsize))
        fontface.set("cap-height", str(caps))
        fontface.set("x-height", str(xheight))
        fontface.set("ascent", str(ascender))
        fontface.set("descent", str(descender))

        for group in self.svg.findall('svg:g'):
            label = group.label
            if "GlyphLayer-" in label:
                unicode_char = label.split("GlyphLayer-")[1]
                glyph = font.get_or_create("svg:glyph[@unicode='{}']".format(unicode_char), Glyph)
                glyph.set("unicode", unicode_char)

                ############################
                # Option 1:
                # Using clone (svg:use) as childnode of svg:glyph

                # use = glyph.get_or_create('svg:use', UseElement)
                # use.set('xlink:href', "#"+group.get("id"))
                # TODO: This creates <use> nodes; why do they do not render on svg fonts dialog?

                ############################
                # Option 2:
                # Using svg:paths as childnodes of svg:glyph

                # for p in group.findall('svg:path'):
                #    d = p.get("d")
                #    d = self.flip_cordinate_system(d, emsize, baseline)
                #    path = glyph.add(PathElement())
                #    path.set("d", d)

                ############################
                # Option 3:
                # Using curve description in d attribute of svg:glyph

                path_d = ""
                for path in group.findall('svg:path'):
                    path_d += " " + self.flip_cordinate_system(path, emsize, baseline)
                glyph.set("d", path_d)

if __name__ == '__main__':
    LayersToSvgFont().run()



#!/usr/bin/env python
# coding=utf-8
"""
Test elements extra logic from svg xml lxml custom classes.
"""

from ..utils import PY3
from ..inx import InxFile

INTERNAL_ARGS = ('help', 'output', 'id', 'selected-nodes')
ARG_TYPES = {
    'Boolean': 'bool',
    'Color': 'color',
    'str': 'string',
    'int': 'int',
    'float': 'float',
}

class InxMixin(object):
    """Tools for Testing INX files, use as a mixin class:

    class MyTests(InxMixin, TestCase):
        def test_inx_file(self):
            self.assertInxIsGood("some_inx_file.inx")
    """
    def assertInxIsGood(self, inx_file): # pylint: disable=invalid-name
        """Test the inx file for consistancy and correctness"""
        self.assertTrue(PY3, "INX files can only be tested in python3")

        inx = InxFile(inx_file)
        if 'help' in inx.ident or inx.script.get('interpreter', None) != 'python':
            return
        cls = inx.extension_class
        # Check class can be matched in python file
        self.assertTrue(cls, 'Can not find class for {}'.format(inx.filename))
        # Check name is reasonable for the class
        if not cls.multi_inx:
            self.assertEqual(
                cls.__name__, inx.slug,
                "Name of extension class {}.{} is different from ident {}".format(
                    cls.__module__, cls.__name__, inx.slug))
            self.assertParams(inx, cls)

    def assertParams(self, inx, cls): # pylint: disable=invalid-name
        """Confirm the params in the inx match the python script"""
        params = dict([(param.name, self.parse_param(param)) for param in inx.params])
        args = dict(self.introspect_arg_parser(cls().arg_parser))
        mismatch_a = list(set(params) ^ set(args) & set(params))
        mismatch_b = list(set(args) ^ set(params) & set(args))
        self.assertFalse(mismatch_a, "{}: Inx params missing from arg parser".format(inx.filename))
        self.assertFalse(mismatch_b, "{}: Script args missing from inx xml".format(inx.filename))

        for param in args:
            if params[param]['type'] and args[param]['type']:
                self.assertEqual(
                    params[param]['type'],
                    args[param]['type'],
                    "Type is not the same for {}:param:{}".format(inx.filename, param))

    def introspect_arg_parser(self, arg_parser):
        """Pull apart the arg parser to find out what we have in it"""
        for action in arg_parser._optionals._actions: # pylint: disable=protected-access
            for opt in action.option_strings:
                # Ignore params internal to inkscape (thus not in the inx)
                if opt.startswith('--') and opt[2:] not in INTERNAL_ARGS:
                    yield (opt[2:], self.introspect_action(action))

    @staticmethod
    def introspect_action(action):
        """Pull apart a single action to get at the juicy insides"""
        return {
            'type': ARG_TYPES.get((action.type or str).__name__, 'string'),
            'default': action.default,
            'choices': action.choices,
            'help': action.help,
        }

    @staticmethod
    def parse_param(param):
        """Pull apart the param element in the inx file"""
        if param.param_type in ('optiongroup', 'notebook'):
            options = param.options
            return {
                'type': None,
                'choices': options,
                'default': options and options[0] or None,
            }
        param_type = param.param_type
        if param.param_type in ('path',):
            param_type = 'string'
        return {
            'type': param_type,
            'default': param.text,
            'choices': None,
        }



#!/usr/bin/env python
"""Adjust all the HSL values"""

import random
import inkex

class HslAdjust(inkex.ColorExtension):
    """Modify the HSL levels of each color"""
    def add_arguments(self, pars):
        pars.add_argument("--tab")
        pars.add_argument("-x", "--hue", type=int, default=0, help="Adjust hue")
        pars.add_argument("-s", "--saturation", type=int, default=0, help="Adjust saturation")
        pars.add_argument("-l", "--lightness", type=int, default=0, help="Adjust lightness")
        pars.add_argument("--random_h", type=inkex.Boolean, dest="random_hue")
        pars.add_argument("--random_s", type=inkex.Boolean, dest="random_saturation")
        pars.add_argument("--random_l", type=inkex.Boolean, dest="random_lightness")

    def modify_color(self, name, color):
        if self.options.random_hue:
            color.hue = int(random.random() * 255.0)
        elif self.options.hue:
            color.hue += (self.options.hue * 2.55)

        if self.options.random_saturation:
            color.saturation = int(random.random() * 255.0)
        elif self.options.saturation:
            color.saturation += (self.options.saturation * 2.55)

        if self.options.random_lightness:
            color.lightness = int(random.random() * 255.0)
        elif self.options.lightness:
            color.lightness += (self.options.lightness * 2.55)

        return color

if __name__ == '__main__':
    HslAdjust().run()



#!/usr/bin/env python
# coding=utf-8
"""
see #inkscape on Freenode and
https://github.com/nikitakit/svg2sif/blob/master/synfig_prepare.py#L370
for an example how to do the transform of parent to children.
"""

import inkex
from inkex import (
    Group, Anchor, Switch, NamedView, Defs, Metadata, ForeignObject,
    ClipPath, Use, SvgDocumentElement,
)

class UngroupDeep(inkex.EffectExtension):
    def add_arguments(self, pars):
        pars.add_argument("--startdepth", type=int, default=0,
                          help="starting depth for ungrouping")
        pars.add_argument("--maxdepth", type=int, default=65535,
                          help="maximum ungrouping depth")
        pars.add_argument("--keepdepth", type=int, default=0,
                          help="levels of ungrouping to leave untouched")

    @staticmethod
    def _merge_style(node, style):
        """Propagate style and transform to remove inheritance
        Originally from
        https://github.com/nikitakit/svg2sif/blob/master/synfig_prepare.py#L370
        """

        # Compose the style attribs
        this_style = node.style
        remaining_style = {}  # Style attributes that are not propagated

        # Filters should remain on the top ancestor
        non_propagated = ["filter"]
        for key in non_propagated:
            if key in this_style.keys():
                remaining_style[key] = this_style[key]
                del this_style[key]

        # Create a copy of the parent style, and merge this style into it
        parent_style_copy = style.copy()
        parent_style_copy.update(this_style)
        this_style = parent_style_copy

        # Merge in any attributes outside of the style
        style_attribs = ["fill", "stroke"]
        for attrib in style_attribs:
            if node.get(attrib):
                this_style[attrib] = node.get(attrib)
                del node.attrib[attrib]

        if isinstance(node, (SvgDocumentElement, Anchor, Group, Switch)):
            # Leave only non-propagating style attributes
            if not remaining_style:
                if "style" in node.keys():
                    del node.attrib["style"]
            else:
                node.style = remaining_style

        else:
            # This element is not a container

            # Merge remaining_style into this_style
            this_style.update(remaining_style)

            # Set the element's style attribs
            node.style = this_style

    def _merge_clippath(self, node, clippathurl):
        if clippathurl and clippathurl != "none":
            node_transform = node.transform
            if node_transform:
                # Clip-paths on nodes with a transform have the transform
                # applied to the clipPath as well, which we don't want.  So, we
                # create new clipPath element with references to all existing
                # clippath subelements, but with the inverse transform applied
                new_clippath = self.svg.defs.add(ClipPath(clipPathUnits='userSpaceOnUse'))
                new_clippath.set_random_id('clipPath')
                clippath = self.svg.getElementById(clippathurl[5:-1])
                for child in clippath.iterchildren():
                    new_clippath.add(Use.new(child, 0, 0))

                # Set the clippathurl to be the one with the inverse transform
                clippathurl = "url(#" + new_clippath.get("id") + ")"

            # Reference the parent clip-path to keep clipping intersection
            # Find end of clip-path chain and add reference there
            node_clippathurl = node.get("clip-path")
            while node_clippathurl:
                node = self.svg.getElementById(node_clippathurl[5:-1])
                node_clippathurl = node.get("clip-path")
            node.set("clip-path", clippathurl)

    # Flatten a group into same z-order as parent, propagating attribs
    def _ungroup(self, node):
        node_parent = node.getparent()
        node_index = list(node_parent).index(node)
        node_style = node.style

        node_transform = node.transform
        node_clippathurl = node.get('clip-path')
        for child in reversed(list(node)):

            child.transform = node_transform * child.transform

            if node.get("style") is not None:
                self._merge_style(child, node_style)
            self._merge_clippath(child, node_clippathurl)
            node_parent.insert(node_index, child)
        node_parent.remove(node)

    # Put all ungrouping restrictions here
    def _want_ungroup(self, node, depth, height):
        if (isinstance(node, Group) and
                node.getparent() is not None and
                height > self.options.keepdepth and
                self.options.startdepth <= depth <=
                self.options.maxdepth):
            return True
        return False

    def _deep_ungroup(self, node):
        # using iteration instead of recursion to avoid hitting Python
        # max recursion depth limits, which is a problem in converted PDFs

        # Seed the queue (stack) with initial node
        q = [{'node': node,
              'depth': 0,
              'prev': {'height': None},
              'height': None}]

        while q:
            current = q[-1]
            node = current['node']
            depth = current['depth']
            height = current['height']

            # Recursion path
            if height is None:
                # Don't enter non-graphical portions of the document
                if isinstance(node, (NamedView, Defs, Metadata, ForeignObject)):
                    q.pop()

                # Base case: Leaf node
                if not isinstance(node, Group) or not list(node):
                    current['height'] = 0

                # Recursive case: Group element with children
                else:
                    depth += 1
                    for child in node.iterchildren():
                        q.append({'node': child, 'prev': current,
                                  'depth': depth, 'height': None})

            # Return path
            else:
                # Ungroup if desired
                if self._want_ungroup(node, depth, height):
                    self._ungroup(node)

                # Propagate (max) height up the call chain
                height += 1
                previous = current['prev']
                prev_height = previous['height']
                if prev_height is None or prev_height < height:
                    previous['height'] = height

                # Only process each node once
                q.pop()

    def effect(self):
        if self.svg.selected:
            for node in self.svg.selected.values():
                self._deep_ungroup(node)
        else:
            for node in self.document.getroot():
                self._deep_ungroup(node)


if __name__ == '__main__':
    UngroupDeep().run()



#!/usr/bin/env python
#
# Copyright 2008, 2009 Hannes Hochreiner
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.
#
"""Effect to add key bindings to jessyInk slide show"""

import inkex
from inkex import Group, Script
from jessyink_install import JessyInkMixin

KEY_CODES = ('LEFT', 'RIGHT', 'DOWN', 'UP', 'HOME', 'END',
             'ENTER', 'SPACE', 'PAGE_UP', 'PAGE_DOWN', 'ESCAPE')

class KeyBindings(JessyInkMixin, inkex.EffectExtension):
    """Add key bindings to slide show"""
    modes = ('slide', 'index', 'drawing')

    def set_options(self, namespace, opt_str, value):
        """Sort through all the options and combine them"""
        slot, action = opt_str.split('_', 1)
        keycodes = getattr(namespace, f"{slot}KeyCodes", {})
        charcodes = getattr(namespace, f"{slot}CharCodes", {})
        if value:
            for val in value.split(","):
                val = val.strip()
                if val in KEY_CODES:
                    keycodes[val + "_KEY"] = self.actions[slot][action]
                elif len(val) == 1:
                    charcodes[val] = self.actions[slot][action]
        setattr(namespace, f"{slot}KeyCodes", keycodes)
        setattr(namespace, f"{slot}CharCodes", charcodes)

    actions = {
        'slide': {
            "export": "slideUpdateExportLayer();",
            "addSlide": "slideAddSlide(activeSlide);",
            "resetTimer": "slideResetTimer();",
            "setDuration": "slideQueryDuration();",
            "backWithEffects": "dispatchEffects(-1);",
            "nextWithEffects": "dispatchEffects(1);",
            "backWithoutEffects": "skipEffects(-1);",
            "nextWithoutEffects": "skipEffects(1);",
            "switchToIndexMode": "toggleSlideIndex();",
            "switchToDrawingMode": "slideSwitchToDrawingMode();",
            "toggleProgressBar": "slideToggleProgressBarVisibility();",
            "firstSlide": "slideSetActiveSlide(0);",
            "lastSlide": "slideSetActiveSlide(slides.length - 1);",
        },
        'drawing': {
            "undo": "drawingUndo();",
            "switchToSlideMode": "drawingSwitchToSlideMode();",
            "pathWidthDefault": "drawingResetPathWidth();",
            "pathWidth1": "drawingSetPathWidth(1.0);",
            "pathWidth3": "drawingSetPathWidth(3.0);",
            "pathWidth5": "drawingSetPathWidth(5.0);",
            "pathWidth7": "drawingSetPathWidth(7.0);",
            "pathWidth9": "drawingSetPathWidth(9.0);",
            "pathColourBlue": "drawingSetPathColour(\"blue\");",
            "pathColourCyan": "drawingSetPathColour(\"cyan\");",
            "pathColourGreen": "drawingSetPathColour(\"green\");",
            "pathColourBlack": "drawingSetPathColour(\"black\");",
            "pathColourMagenta": "drawingSetPathColour(\"magenta\");",
            "pathColourOrange": "drawingSetPathColour(\"orange\");",
            "pathColourRed": "drawingSetPathColour(\"red\");",
            "pathColourWhite": "drawingSetPathColour(\"white\");",
            "pathColourYellow": "drawingSetPathColour(\"yellow\");",
        },
        'index': {
            "selectSlideToLeft": "indexSetPageSlide(activeSlide - 1);",
            "selectSlideToRight": "indexSetPageSlide(activeSlide + 1);",
            "selectSlideAbove": "indexSetPageSlide(activeSlide - INDEX_COLUMNS);",
            "selectSlideBelow": "indexSetPageSlide(activeSlide + INDEX_COLUMNS);",
            "previousPage": "indexSetPageSlide(activeSlide - INDEX_COLUMNS * INDEX_COLUMNS);",
            "nextPage": "indexSetPageSlide(activeSlide + INDEX_COLUMNS * INDEX_COLUMNS);",
            "firstSlide": "indexSetPageSlide(0);",
            "lastSlide": "indexSetPageSlide(slides.length - 1);",
            "switchToSlideMode": "toggleSlideIndex();",
            "decreaseNumberOfColumns": "indexDecreaseNumberOfColumns();",
            "increaseNumberOfColumns": "indexIncreaseNumberOfColumns();",
            "setNumberOfColumnsToDefault": "indexResetNumberOfColumns();",
        }
    }

    def add_arguments(self, pars):
        pars.add_argument('--tab')
        for slot, actions in self.actions.items():
            for action in actions:
                pars.add_argument(f'--{slot}_{action}')

    def effect(self):
        self.is_installed()

        for name in list(self.options.__dict__):
            if '_' in name:
                self.set_options(self.options, name, self.options.__dict__.pop(name))

        # Remove old master slide property
        for node in self.svg.xpath("//svg:g[@jessyink:customKeyBindings='customKeyBindings']"):
            node.delete()

        # Set custom key bindings.
        node_text = """function getCustomKeyBindingsSub()
{
    var keyDict = new Object();
    keyDict[SLIDE_MODE] = new Object();
    keyDict[INDEX_MODE] = new Object();
    keyDict[DRAWING_MODE] = new Object();
"""

        for key, value in self.options.slideKeyCodes.items():
            node_text += f"    keyDict[SLIDE_MODE][{key}] = function() {{ {value} }};\n"

        for key, value in self.options.drawingKeyCodes.items():
            node_text += f"    keyDict[DRAWING_MODE][{key}] = function() {{ {value} }};\n"

        for key, value in self.options.indexKeyCodes.items():
            node_text += f"    keyDict[INDEX_MODE][{key}] = function() {{ {value} }};\n"

        # Set custom char bindings.
        node_text += """    return keyDict;
}

function getCustomCharBindingsSub()
{
    var charDict = new Object();
    charDict[SLIDE_MODE] = new Object();
    charDict[INDEX_MODE] = new Object();
    charDict[DRAWING_MODE] = new Object();
"""

        for key, value in self.options.slideCharCodes.items():
            node_text += f'    charDict[SLIDE_MODE]["{key}"] = function() {{ {value} }};\n'

        for key, value in self.options.drawingCharCodes.items():
            node_text += f'    charDict[DRAWING_MODE]["{key}"] = function() {{ {value} }};\n'

        for key, value in self.options.indexCharCodes.items():
            node_text += f'    charDict[INDEX_MODE]["{key}"] = function() {{ {value} }};\n'

        node_text += "    return charDict;" + "\n"
        node_text += "}" + "\n"

        # Create new script node
        group = self.svg.add(Group())
        script = group.add(Script())
        script.text = node_text
        group.set("jessyink:customKeyBindings", "customKeyBindings")
        group.set("onload", "this.getCustomCharBindings = function() { "\
            "return getCustomCharBindingsSub(); }; "\
            "this.getCustomKeyBindings = function() { return getCustomKeyBindingsSub(); };")


if __name__ == '__main__':
    KeyBindings().run()



#!/usr/bin/env python
#
# Copyright 2008, 2009 Hannes Hochreiner
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.
#
"""Add mouse handler for jessyInk"""

import inkex
from inkex.elements import BaseElement, Script
from jessyink_install import JessyInkMixin

class MouseHandler(BaseElement):
    """jessyInk mouse handler"""
    tag_name = 'jessyink:mousehandler'

class AddMouseHandler(JessyInkMixin, inkex.EffectExtension):
    """Add mouse handler"""
    def add_arguments(self, pars):
        pars.add_argument('--tab')
        pars.add_argument('--mouseSetting', default='default')

    def effect(self):
        self.is_installed()
        # Remove old mouse handler
        for node in self.svg.xpath("//jessyink:mousehandler"):
            node.delete()

        # Create new script node.
        script = Script()
        group = MouseHandler()

        if self.options.mouseSetting == "noclick":
            name = "noclick"
        elif self.options.mouseSetting == "draggingZoom":
            name = "zoomControl"
        elif self.options.mouseSetting == "default":
            # Default is to remove script and continue
            return

        with open(self.get_resource(f"jessyInk_core_mouseHandler_{name}.js")) as fhl:
            script.text = fhl.read()
        group.set("jessyink:subtype", f"jessyInk_core_mouseHandler_{name}")

        group.append(script)
        self.svg.append(group)

if __name__ == '__main__':
    AddMouseHandler().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2008 Stephen Silver
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
"""
Simple wrapper around ps2pdf
"""

import inkex
from inkex.command import call

class PostscriptInput(inkex.CallExtension):
    """Load Postscript/EPS Files by calling ps2pdf program"""
    input_ext = 'ps'
    output_ext = 'pdf'
    multi_inx = True

    def add_arguments(self, pars):
        pars.add_argument('--crop', type=inkex.Boolean, default=False)

    def call(self, input_file, output_file):
        crop = '-dEPSCrop' if self.options.crop else None
        call('ps2pdf', crop, input_file, output_file)

if __name__ == '__main__':
    PostscriptInput().run()
